@@ -28,8 +28,8 @@
 #   $ ./checkcompatibility.py ${SOURCE_GIT_REVISION} ${GIT_BRANCH_OR_TAG}
 # or with some options:
 #   $ ./dev-support/checkcompatibility.py \
-#      --annotation org.apache.hadoop.hbase.classification.InterfaceAudience.Public \
-#      --annotation org.apache.hadoop.hbase.classification.InterfaceAudience.LimitedPrivate \
+#      --annotation org.apache.yetus.audience.InterfaceAudience.Public \
+#      --annotation org.apache.yetus.audience.InterfaceAudience.LimitedPrivate \
 #      --include-file "hbase-*" \
 #      --known_problems_path ~/known_problems.json \
 #      rel/1.0.0 branch-1.2
@@ -270,8 +270,8 @@ def log_java_acc_version():
     java_acc_path = os.path.join(
         get_java_acc_dir(), "japi-compliance-checker.pl")
 
-    args = ["perl", java_acc_path, "-version"]
-    logging.info(check_output(args))
+    args = ["perl", java_acc_path, "-dumpversion"]
+    logging.info("Java ACC version: " + check_output(args))
 
 def run_java_acc(src_name, src_jars, dst_name, dst_jars, annotations, skip_annotations, name):
     """ Run the compliance checker to compare 'src' and 'dst'. """
