@@ -17,32 +17,34 @@
  */
 package org.apache.hadoop.hbase.client;
 
+import static org.apache.hadoop.hbase.client.ConnectionUtils.calcEstimatedSize;
 import static org.apache.hadoop.hbase.client.ConnectionUtils.createScanResultCache;
+import static org.apache.hadoop.hbase.client.ConnectionUtils.incRegionCountMetrics;
 
 import java.io.IOException;
 import java.io.InterruptedIOException;
-import java.util.LinkedList;
+import java.util.ArrayDeque;
+import java.util.Queue;
 import java.util.concurrent.ExecutorService;
-
-import org.apache.commons.lang.mutable.MutableBoolean;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.commons.lang3.mutable.MutableBoolean;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.HRegionInfo;
-import org.apache.hadoop.hbase.regionserver.LeaseException;
 import org.apache.hadoop.hbase.NotServingRegionException;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.UnknownScannerException;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
 import org.apache.hadoop.hbase.client.ScannerCallable.MoreResults;
 import org.apache.hadoop.hbase.exceptions.OutOfOrderScannerNextException;
 import org.apache.hadoop.hbase.exceptions.ScannerResetException;
 import org.apache.hadoop.hbase.ipc.RpcControllerFactory;
-import org.apache.hadoop.hbase.protobuf.ProtobufUtil;
+import org.apache.hadoop.hbase.regionserver.LeaseException;
 import org.apache.hadoop.hbase.regionserver.RegionServerStoppedException;
+import org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Implements the scanner interface for the HBase client. If there are multiple regions in a table,
@@ -50,7 +52,8 @@ import org.apache.hadoop.hbase.util.Bytes;
  */
 @InterfaceAudience.Private
 public abstract class ClientScanner extends AbstractClientScanner {
-  private static final Log LOG = LogFactory.getLog(ClientScanner.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(ClientScanner.class);
 
   protected final Scan scan;
   protected boolean closed = false;
@@ -58,7 +61,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
   // wonky: e.g. if it splits on us.
   protected HRegionInfo currentRegion = null;
   protected ScannerCallableWithReplicas callable = null;
-  protected final LinkedList<Result> cache = new LinkedList<Result>();
+  protected Queue<Result> cache;
   private final ScanResultCache scanResultCache;
   protected final int caching;
   protected long lastNext;
@@ -66,7 +69,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
   protected Result lastResult = null;
   protected final long maxScannerResultSize;
   private final ClusterConnection connection;
-  private final TableName tableName;
+  protected final TableName tableName;
   protected final int scannerTimeout;
   protected boolean scanMetricsPublished = false;
   protected RpcRetryingCaller<Result[]> caller;
@@ -130,24 +133,20 @@ public abstract class ClientScanner extends AbstractClientScanner {
     this.rpcControllerFactory = controllerFactory;
 
     this.conf = conf;
-    this.scanResultCache = createScanResultCache(scan, cache);
+
+    this.scanResultCache = createScanResultCache(scan);
+    initCache();
+  }
+
+  protected final int getScanReplicaId() {
+    return scan.getReplicaId() >= RegionReplicaUtil.DEFAULT_REPLICA_ID ? scan.getReplicaId() :
+      RegionReplicaUtil.DEFAULT_REPLICA_ID;
   }
 
   protected ClusterConnection getConnection() {
     return this.connection;
   }
 
-  /**
-   * @return Table name
-   * @deprecated As of release 0.96
-   *             (<a href="https://issues.apache.org/jira/browse/HBASE-9508">HBASE-9508</a>). This
-   *             will be removed in HBase 2.0.0. Use {@link #getTable()}.
-   */
-  @Deprecated
-  protected byte[] getTableName() {
-    return this.tableName.getName();
-  }
-
   protected TableName getTable() {
     return this.tableName;
   }
@@ -248,9 +247,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
         new ScannerCallableWithReplicas(getTable(), getConnection(), createScannerCallable(), pool,
             primaryOperationTimeout, scan, getRetries(), scannerTimeout, caching, conf, caller);
     this.callable.setCaching(this.caching);
-    if (this.scanMetrics != null) {
-      this.scanMetrics.countOfRegions.incrementAndGet();
-    }
+    incRegionCountMetrics(scanMetrics);
     return true;
   }
 
@@ -293,23 +290,31 @@ public abstract class ClientScanner extends AbstractClientScanner {
     scanMetricsPublished = true;
   }
 
-  @Override
-  public Result next() throws IOException {
-    // If the scanner is closed and there's nothing left in the cache, next is a no-op.
-    if (cache.size() == 0 && this.closed) {
+  protected void initSyncCache() {
+    cache = new ArrayDeque<>();
+  }
+
+  protected Result nextWithSyncCache() throws IOException {
+    Result result = cache.poll();
+    if (result != null) {
+      return result;
+    }
+    // If there is nothing left in the cache and the scanner is closed,
+    // return a no-op
+    if (this.closed) {
       return null;
     }
-    if (cache.size() == 0) {
-      loadCache();
-    }
 
-    if (cache.size() > 0) {
-      return cache.poll();
-    }
+    loadCache();
+
+    // try again to load from cache
+    result = cache.poll();
 
     // if we exhausted this scanner before calling close, write out the scan metrics
-    writeScanMetrics();
-    return null;
+    if (result == null) {
+      writeScanMetrics();
+    }
+    return result;
   }
 
   public int getCacheSize() {
@@ -406,10 +411,9 @@ public abstract class ClientScanner extends AbstractClientScanner {
     long remainingResultSize = maxScannerResultSize;
     int countdown = this.caching;
     // This is possible if we just stopped at the boundary of a region in the previous call.
-    if (callable == null) {
-      if (!moveToNextRegion()) {
-        return;
-      }
+    if (callable == null && !moveToNextRegion()) {
+      closed = true;
+      return;
     }
     // This flag is set when we want to skip the result returned. We do
     // this when we reset scanner because it split under us.
@@ -454,15 +458,17 @@ public abstract class ClientScanner extends AbstractClientScanner {
       // Results to the scanner's cache. If partial results are not allowed to be seen by the
       // caller, all book keeping will be performed within this method.
       int numberOfCompleteRowsBefore = scanResultCache.numberOfCompleteRows();
-      scanResultCache.loadResultsToCache(values, callable.isHeartbeatMessage());
+      Result[] resultsToAddToCache =
+          scanResultCache.addAndGet(values, callable.isHeartbeatMessage());
       int numberOfCompleteRows =
           scanResultCache.numberOfCompleteRows() - numberOfCompleteRowsBefore;
-      if (scanResultCache.getCount() > 0) {
-        remainingResultSize -= scanResultCache.getResultSize();
-        scanResultCache.resetResultSize();
-        countdown -= scanResultCache.getCount();
-        scanResultCache.resetCount();
-        this.lastResult = scanResultCache.getLastResult();
+      for (Result rs : resultsToAddToCache) {
+        cache.add(rs);
+        long estimatedHeapSizeOfResult = calcEstimatedSize(rs);
+        countdown--;
+        remainingResultSize -= estimatedHeapSizeOfResult;
+        addEstimatedSize(estimatedHeapSizeOfResult);
+        this.lastResult = rs;
       }
 
       if (scan.getLimit() > 0) {
@@ -470,7 +476,7 @@ public abstract class ClientScanner extends AbstractClientScanner {
         assert newLimit >= 0;
         scan.setLimit(newLimit);
       }
-      if (scanExhausted(values)) {
+      if (scan.getLimit() == 0 || scanExhausted(values)) {
         closeScanner();
         closed = true;
         break;
@@ -482,10 +488,8 @@ public abstract class ClientScanner extends AbstractClientScanner {
           // processing of the scan is taking a long time server side. Rather than continue to
           // loop until a limit (e.g. size or caching) is reached, break out early to avoid causing
           // unnecesary delays to the caller
-          if (LOG.isTraceEnabled()) {
-            LOG.trace("Heartbeat message received and cache contains Results." +
-                " Breaking out of scan loop");
-          }
+          LOG.trace("Heartbeat message received and cache contains Results. " +
+            "Breaking out of scan loop");
           // we know that the region has not been exhausted yet so just break without calling
           // closeScannerIfExhausted
           break;
@@ -524,12 +528,21 @@ public abstract class ClientScanner extends AbstractClientScanner {
       // we are done with the current region
       if (regionExhausted) {
         if (!moveToNextRegion()) {
+          closed = true;
           break;
         }
       }
     }
   }
 
+  protected void addEstimatedSize(long estimatedHeapSizeOfResult) {
+    return;
+  }
+
+  public int getCacheCount() {
+    return cache != null ? cache.size() : 0;
+  }
+
   @Override
   public void close() {
     if (!scanMetricsPublished) writeScanMetrics();
@@ -541,9 +554,10 @@ public abstract class ClientScanner extends AbstractClientScanner {
         // We used to catch this error, interpret, and rethrow. However, we
         // have since decided that it's not nice for a scanner's close to
         // throw exceptions. Chances are it was just due to lease time out.
+        LOG.debug("scanner failed to close", e);
       } catch (IOException e) {
         /* An exception other than UnknownScanner is unexpected. */
-        LOG.warn("scanner failed to close. Exception follows: " + e);
+        LOG.warn("scanner failed to close.", e);
       }
       callable = null;
     }
@@ -552,18 +566,28 @@ public abstract class ClientScanner extends AbstractClientScanner {
 
   @Override
   public boolean renewLease() {
-    if (callable != null) {
-      // do not return any rows, do not advance the scanner
-      callable.setRenew(true);
-      try {
-        this.caller.callWithoutRetries(callable, this.scannerTimeout);
-      } catch (Exception e) {
-        return false;
-      } finally {
-        callable.setRenew(false);
-      }
-      return true;
+    if (callable == null) {
+      return false;
     }
-    return false;
+    // do not return any rows, do not advance the scanner
+    callable.setRenew(true);
+    try {
+      this.caller.callWithoutRetries(callable, this.scannerTimeout);
+      return true;
+    } catch (Exception e) {
+      LOG.debug("scanner failed to renew lease", e);
+      return false;
+    } finally {
+      callable.setRenew(false);
+    }
+  }
+
+  protected void initCache() {
+    initSyncCache();
+  }
+
+  @Override
+  public Result next() throws IOException {
+    return nextWithSyncCache();
   }
 }
