@@ -34,14 +34,15 @@ import java.util.NoSuchElementException;
 import java.util.TreeMap;
 
 import org.apache.hadoop.hbase.Cell;
+import org.apache.hadoop.hbase.CellComparator;
 import org.apache.hadoop.hbase.CellScannable;
 import org.apache.hadoop.hbase.CellScanner;
 import org.apache.hadoop.hbase.CellUtil;
+import org.apache.hadoop.hbase.HConstants;
+import org.apache.hadoop.hbase.PrivateCellUtil;
 import org.apache.hadoop.hbase.KeyValue;
 import org.apache.hadoop.hbase.KeyValueUtil;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
-import org.apache.hadoop.hbase.classification.InterfaceStability;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos;
+import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.hadoop.hbase.util.Bytes;
 
 /**
@@ -61,7 +62,8 @@ import org.apache.hadoop.hbase.util.Bytes;
  * To get a mapping of qualifiers to latest values for an individual family use
  * {@link #getFamilyMap(byte[])}.<p>
  *
- * To get the latest value for a specific family and qualifier use {@link #getValue(byte[], byte[])}.
+ * To get the latest value for a specific family and qualifier use
+ * {@link #getValue(byte[], byte[])}.
  *
  * A Result is backed by an array of {@link Cell} objects, each representing
  * an HBase cell defined by the row, family, qualifier, timestamp, and value.<p>
@@ -78,7 +80,6 @@ import org.apache.hadoop.hbase.util.Bytes;
  * in then use {@link #copyFrom(Result)}
  */
 @InterfaceAudience.Public
-@InterfaceStability.Stable
 public class Result implements CellScannable, CellScanner {
   private Cell[] cells;
   private Boolean exists; // if the query was just to check existence.
@@ -92,9 +93,10 @@ public class Result implements CellScannable, CellScanner {
   // that this is where we cache row if we're ever asked for it.
   private transient byte [] row = null;
   // Ditto for familyMap.  It can be composed on fly from passed in kvs.
-  private transient NavigableMap<byte[], NavigableMap<byte[], NavigableMap<Long, byte[]>>> familyMap = null;
+  private transient NavigableMap<byte[], NavigableMap<byte[], NavigableMap<Long, byte[]>>>
+      familyMap = null;
 
-  private static ThreadLocal<byte[]> localBuffer = new ThreadLocal<byte[]>();
+  private static ThreadLocal<byte[]> localBuffer = new ThreadLocal<>();
   private static final int PAD_WIDTH = 128;
   public static final Result EMPTY_RESULT = new Result(true);
 
@@ -104,7 +106,7 @@ public class Result implements CellScannable, CellScanner {
    * Index for where we are when Result is acting as a {@link CellScanner}.
    */
   private int cellScannerIndex = INITIAL_CELLSCANNER_INDEX;
-  private ClientProtos.RegionLoadStats stats;
+  private RegionLoadStats stats;
 
   private final boolean readonly;
 
@@ -112,9 +114,9 @@ public class Result implements CellScannable, CellScanner {
 
   /**
    * Creates an empty Result w/ no KeyValue payload; returns null if you call {@link #rawCells()}.
-   * Use this to represent no results if <code>null</code> won't do or in old 'mapred' as oppposed to 'mapreduce' package
-   * MapReduce where you need to overwrite a Result
-   * instance with a {@link #copyFrom(Result)} call.
+   * Use this to represent no results if {@code null} won't do or in old 'mapred' as opposed
+   * to 'mapreduce' package MapReduce where you need to overwrite a Result instance with a
+   * {@link #copyFrom(Result)} call.
    */
   public Result() {
     this(false);
@@ -129,23 +131,6 @@ public class Result implements CellScannable, CellScanner {
     this.readonly = readonly;
   }
 
-  /**
-   * @deprecated Use {@link #create(List)} instead.
-   */
-  @Deprecated
-  public Result(KeyValue [] cells) {
-    this(cells, null, false, false);
-  }
-
-  /**
-   * @deprecated Use {@link #create(List)} instead.
-   */
-  @Deprecated
-  public Result(List<KeyValue> kvs) {
-    // TODO: Here we presume the passed in Cells are KVs.  One day this won't always be so.
-    this(kvs.toArray(new Cell[kvs.size()]), null, false, false);
-  }
-
   /**
    * Instantiate a Result with the specified List of KeyValues.
    * <br><strong>Note:</strong> You must ensure that the keyvalues are already sorted.
@@ -163,11 +148,12 @@ public class Result implements CellScannable, CellScanner {
     return create(cells, exists, stale, false);
   }
 
-  public static Result create(List<Cell> cells, Boolean exists, boolean stale, boolean partial) {
+  public static Result create(List<Cell> cells, Boolean exists, boolean stale,
+      boolean mayHaveMoreCellsInRow) {
     if (exists != null){
-      return new Result(null, exists, stale, partial);
+      return new Result(null, exists, stale, mayHaveMoreCellsInRow);
     }
-    return new Result(cells.toArray(new Cell[cells.size()]), null, stale, partial);
+    return new Result(cells.toArray(new Cell[cells.size()]), null, stale, mayHaveMoreCellsInRow);
   }
 
   /**
@@ -216,7 +202,9 @@ public class Result implements CellScannable, CellScanner {
    */
   public byte [] getRow() {
     if (this.row == null) {
-      this.row = this.cells == null || this.cells.length == 0? null: CellUtil.cloneRow(this.cells[0]);
+      this.row = (this.cells == null || this.cells.length == 0) ?
+          null :
+          CellUtil.cloneRow(this.cells[0]);
     }
     return this.row;
   }
@@ -225,14 +213,14 @@ public class Result implements CellScannable, CellScanner {
    * Return the array of Cells backing this Result instance.
    *
    * The array is sorted from smallest -&gt; largest using the
-   * {@link KeyValue#COMPARATOR}.
+   * {@link CellComparator}.
    *
    * The array only contains what your Get or Scan specifies and no more.
    * For example if you request column "A" 1 version you will have at most 1
    * Cell in the array. If you request column "A" with 2 version you will
    * have at most 2 Cells, with the first one being the newer timestamp and
    * the second being the older timestamp (this is the sort order defined by
-   * {@link KeyValue#COMPARATOR}).  If columns don't exist, they won't be
+   * {@link CellComparator}).  If columns don't exist, they won't be
    * present in the result. Therefore if you ask for 1 version all columns,
    * it is safe to iterate over this array and expect to see 1 Cell for
    * each column and no more.
@@ -245,25 +233,6 @@ public class Result implements CellScannable, CellScanner {
     return cells;
   }
 
-  /**
-   * Return an cells of a Result as an array of KeyValues
-   *
-   * WARNING do not use, expensive.  This does an arraycopy of the cell[]'s value.
-   *
-   * Added to ease transition from  0.94 -&gt; 0.96.
-   *
-   * @deprecated as of 0.96, use {@link #rawCells()}
-   * @return array of KeyValues, empty array if nothing in result.
-   */
-  @Deprecated
-  public KeyValue[] raw() {
-    KeyValue[] kvs = new KeyValue[cells.length];
-    for (int i = 0 ; i < kvs.length; i++) {
-      kvs[i] = KeyValueUtil.ensureKeyValue(cells[i]);
-    }
-    return kvs;
-  }
-
   /**
    * Create a sorted list of the Cell's in this result.
    *
@@ -275,32 +244,9 @@ public class Result implements CellScannable, CellScanner {
     return isEmpty()? null: Arrays.asList(rawCells());
   }
 
-  /**
-   * Return an cells of a Result as an array of KeyValues
-   *
-   * WARNING do not use, expensive.  This does  an arraycopy of the cell[]'s value.
-   *
-   * Added to ease transition from  0.94 -&gt; 0.96.
-   *
-   * @deprecated as of 0.96, use {@link #listCells()}
-   * @return all sorted List of KeyValues; can be null if no cells in the result
-   */
-  @Deprecated
-  public List<KeyValue> list() {
-    return isEmpty() ? null : Arrays.asList(raw());
-  }
-
-  /**
-   * @deprecated Use {@link #getColumnCells(byte[], byte[])} instead.
-   */
-  @Deprecated
-  public List<KeyValue> getColumn(byte [] family, byte [] qualifier) {
-    return KeyValueUtil.ensureKeyValues(getColumnCells(family, qualifier));
-  }
-
   /**
    * Return the Cells for the specific column.  The Cells are sorted in
-   * the {@link KeyValue#COMPARATOR} order.  That implies the first entry in
+   * the {@link CellComparator} order.  That implies the first entry in
    * the list is the most recent column.  If the query (Scan or Get) only
    * requested 1 version the list will contain at most 1 entry.  If the column
    * did not exist in the result set (either the column does not exist
@@ -314,7 +260,7 @@ public class Result implements CellScannable, CellScanner {
    * did not exist in the result set
    */
   public List<Cell> getColumnCells(byte [] family, byte [] qualifier) {
-    List<Cell> result = new ArrayList<Cell>();
+    List<Cell> result = new ArrayList<>();
 
     Cell [] kvs = rawCells();
 
@@ -326,7 +272,7 @@ public class Result implements CellScannable, CellScanner {
       return result; // cant find it
     }
 
-    for (int i = pos ; i < kvs.length ; i++ ) {
+    for (int i = pos; i < kvs.length; i++) {
       if (CellUtil.matchingColumn(kvs[i], family,qualifier)) {
         result.add(kvs[i]);
       } else {
@@ -337,15 +283,27 @@ public class Result implements CellScannable, CellScanner {
     return result;
   }
 
+  private byte[] notNullBytes(final byte[] bytes) {
+    if (bytes == null) {
+      return HConstants.EMPTY_BYTE_ARRAY;
+    } else {
+      return bytes;
+    }
+  }
+
   protected int binarySearch(final Cell [] kvs,
                              final byte [] family,
                              final byte [] qualifier) {
+    byte[] familyNotNull = notNullBytes(family);
+    byte[] qualifierNotNull = notNullBytes(qualifier);
     Cell searchTerm =
-        KeyValueUtil.createFirstOnRow(CellUtil.cloneRow(kvs[0]),
-            family, qualifier);
+        PrivateCellUtil.createFirstOnRow(kvs[0].getRowArray(),
+            kvs[0].getRowOffset(), kvs[0].getRowLength(),
+            familyNotNull, 0, (byte)familyNotNull.length,
+            qualifierNotNull, 0, qualifierNotNull.length);
 
     // pos === ( -(insertion point) - 1)
-    int pos = Arrays.binarySearch(kvs, searchTerm, KeyValue.COMPARATOR);
+    int pos = Arrays.binarySearch(kvs, searchTerm, CellComparator.getInstance());
     // never will exact match
     if (pos < 0) {
       pos = (pos+1) * -1;
@@ -390,7 +348,7 @@ public class Result implements CellScannable, CellScanner {
         qualifier, qoffset, qlength);
 
     // pos === ( -(insertion point) - 1)
-    int pos = Arrays.binarySearch(kvs, searchTerm, KeyValue.COMPARATOR);
+    int pos = Arrays.binarySearch(kvs, searchTerm, CellComparator.getInstance());
     // never will exact match
     if (pos < 0) {
       pos = (pos+1) * -1;
@@ -402,14 +360,6 @@ public class Result implements CellScannable, CellScanner {
     return pos;
   }
 
-  /**
-   * @deprecated Use {@link #getColumnLatestCell(byte[], byte[])} instead.
-   */
-  @Deprecated
-  public KeyValue getColumnLatest(byte [] family, byte [] qualifier) {
-    return KeyValueUtil.ensureKeyValue(getColumnLatestCell(family, qualifier));
-  }
-
   /**
    * The Cell for the most recent timestamp for a given column.
    *
@@ -434,16 +384,6 @@ public class Result implements CellScannable, CellScanner {
     return null;
   }
 
-  /**
-   * @deprecated Use {@link #getColumnLatestCell(byte[], int, int, byte[], int, int)} instead.
-   */
-  @Deprecated
-  public KeyValue getColumnLatest(byte [] family, int foffset, int flength,
-      byte [] qualifier, int qoffset, int qlength) {
-    return KeyValueUtil.ensureKeyValue(
-        getColumnLatestCell(family, foffset, flength, qualifier, qoffset, qlength));
-  }
-
   /**
    * The Cell for the most recent timestamp for a given column.
    *
@@ -468,7 +408,8 @@ public class Result implements CellScannable, CellScanner {
     if (pos == -1) {
       return null;
     }
-    if (CellUtil.matchingColumn(kvs[pos], family, foffset, flength, qualifier, qoffset, qlength)) {
+    if (PrivateCellUtil.matchingColumn(kvs[pos], family, foffset, flength, qualifier, qoffset,
+      qlength)) {
       return kvs[pos];
     }
     return null;
@@ -693,20 +634,18 @@ public class Result implements CellScannable, CellScanner {
     if(isEmpty()) {
       return null;
     }
-    this.familyMap = new TreeMap<byte[], NavigableMap<byte[], NavigableMap<Long, byte[]>>>(Bytes.BYTES_COMPARATOR);
+    this.familyMap = new TreeMap<>(Bytes.BYTES_COMPARATOR);
     for(Cell kv : this.cells) {
       byte [] family = CellUtil.cloneFamily(kv);
-      NavigableMap<byte[], NavigableMap<Long, byte[]>> columnMap =
-        familyMap.get(family);
+      NavigableMap<byte[], NavigableMap<Long, byte[]>> columnMap = familyMap.get(family);
       if(columnMap == null) {
-        columnMap = new TreeMap<byte[], NavigableMap<Long, byte[]>>
-          (Bytes.BYTES_COMPARATOR);
+        columnMap = new TreeMap<>(Bytes.BYTES_COMPARATOR);
         familyMap.put(family, columnMap);
       }
       byte [] qualifier = CellUtil.cloneQualifier(kv);
       NavigableMap<Long, byte[]> versionMap = columnMap.get(qualifier);
       if(versionMap == null) {
-        versionMap = new TreeMap<Long, byte[]>(new Comparator<Long>() {
+        versionMap = new TreeMap<>(new Comparator<Long>() {
           @Override
           public int compare(Long l1, Long l2) {
             return l2.compareTo(l1);
@@ -737,12 +676,10 @@ public class Result implements CellScannable, CellScanner {
     if(isEmpty()) {
       return null;
     }
-    NavigableMap<byte[], NavigableMap<byte[], byte[]>> returnMap =
-      new TreeMap<byte[], NavigableMap<byte[], byte[]>>(Bytes.BYTES_COMPARATOR);
+    NavigableMap<byte[], NavigableMap<byte[], byte[]>> returnMap = new TreeMap<>(Bytes.BYTES_COMPARATOR);
     for(Map.Entry<byte[], NavigableMap<byte[], NavigableMap<Long, byte[]>>>
       familyEntry : familyMap.entrySet()) {
-      NavigableMap<byte[], byte[]> qualifierMap =
-        new TreeMap<byte[], byte[]>(Bytes.BYTES_COMPARATOR);
+      NavigableMap<byte[], byte[]> qualifierMap = new TreeMap<>(Bytes.BYTES_COMPARATOR);
       for(Map.Entry<byte[], NavigableMap<Long, byte[]>> qualifierEntry :
         familyEntry.getValue().entrySet()) {
         byte [] value =
@@ -768,8 +705,7 @@ public class Result implements CellScannable, CellScanner {
     if(isEmpty()) {
       return null;
     }
-    NavigableMap<byte[], byte[]> returnMap =
-      new TreeMap<byte[], byte[]>(Bytes.BYTES_COMPARATOR);
+    NavigableMap<byte[], byte[]> returnMap = new TreeMap<>(Bytes.BYTES_COMPARATOR);
     NavigableMap<byte[], NavigableMap<Long, byte[]>> qualifierMap =
       familyMap.get(family);
     if(qualifierMap == null) {
@@ -842,14 +778,35 @@ public class Result implements CellScannable, CellScanner {
    * @throws Exception Every difference is throwing an exception
    */
   public static void compareResults(Result res1, Result res2)
+    throws Exception{
+    compareResults(res1, res2, true);
+  }
+
+  /**
+   * Does a deep comparison of two Results, down to the byte arrays.
+   * @param res1 first result to compare
+   * @param res2 second result to compare
+   * @param verbose includes string representation for all cells in the exception if true;
+   *                otherwise include rowkey only
+   * @throws Exception Every difference is throwing an exception
+   */
+  public static void compareResults(Result res1, Result res2, boolean verbose)
       throws Exception {
     if (res2 == null) {
       throw new Exception("There wasn't enough rows, we stopped at "
           + Bytes.toStringBinary(res1.getRow()));
     }
     if (res1.size() != res2.size()) {
-      throw new Exception("This row doesn't have the same number of KVs: "
-          + res1.toString() + " compared to " + res2.toString());
+      if (verbose) {
+        throw new Exception(
+          "This row doesn't have the same number of KVs: "
+            + res1 + " compared to " + res2);
+      } else {
+        throw new Exception(
+          "This row doesn't have the same number of KVs: row="
+            + Bytes.toStringBinary(res1.getRow())
+            + ", " + res1.size() + " cells are compared to " + res2.size() + " cells");
+      }
     }
     Cell[] ourKVs = res1.rawCells();
     Cell[] replicatedKVs = res2.rawCells();
@@ -857,8 +814,13 @@ public class Result implements CellScannable, CellScanner {
       if (!ourKVs[i].equals(replicatedKVs[i]) ||
           !CellUtil.matchingValue(ourKVs[i], replicatedKVs[i]) ||
           !CellUtil.matchingTags(ourKVs[i], replicatedKVs[i])) {
-        throw new Exception("This result was different: "
-            + res1.toString() + " compared to " + res2.toString());
+        if (verbose) {
+          throw new Exception("This result was different: "
+            + res1 + " compared to " + res2);
+        } else {
+          throw new Exception("This result was different: row="
+            + Bytes.toStringBinary(res1.getRow()));
+        }
       }
     }
   }
@@ -870,26 +832,11 @@ public class Result implements CellScannable, CellScanner {
    * @return The complete result that is formed by combining all of the partial results together
    * @throws IOException A complete result cannot be formed because the results in the partial list
    *           come from different rows
-   * @deprecated
-   */
-  @Deprecated
-  public static Result createCompleteResult(List<Result> partialResults)
-      throws IOException {
-    return createCompleteResult((Iterable<Result>)partialResults);
-  }
-
-  /**
-   * Forms a single result from the partial results in the partialResults list. This method is
-   * useful for reconstructing partial results on the client side.
-   * @param partialResults iterable of partial results
-   * @return The complete result that is formed by combining all of the partial results together
-   * @throws IOException A complete result cannot be formed because the results in the partial list
-   *           come from different rows
    */
   public static Result createCompleteResult(Iterable<Result> partialResults)
       throws IOException {
     if (partialResults == null) {
-      return Result.create(Collections.<Cell> emptyList(), null, false);
+      return Result.create(Collections.emptyList(), null, false);
     }
     List<Cell> cells = new ArrayList<>();
     boolean stale = false;
@@ -939,7 +886,7 @@ public class Result implements CellScannable, CellScanner {
       return size;
     }
     for (Cell c : result.rawCells()) {
-      size += CellUtil.estimatedHeapSizeOf(c);
+      size += c.heapSize();
     }
     return size;
   }
@@ -1027,27 +974,12 @@ public class Result implements CellScannable, CellScanner {
     return mayHaveMoreCellsInRow;
   }
 
-  /**
-   * Add load information about the region to the information about the result
-   * @param loadStats statistics about the current region from which this was returned
-   * @deprecated use {@link #setStatistics(ClientProtos.RegionLoadStats)} instead
-   * @throws UnsupportedOperationException if invoked on instance of EMPTY_RESULT
-   * (which is supposed to be immutable).
-   */
-  @InterfaceAudience.Private
-  @Deprecated
-  public void addResults(ClientProtos.RegionLoadStats loadStats) {
-    checkReadonly();
-    this.stats = loadStats;
-  }
-
   /**
    * Set load information about the region to the information about the result
    * @param loadStats statistics about the current region from which this was returned
    */
   @InterfaceAudience.Private
-  @Deprecated
-  public void setStatistics(ClientProtos.RegionLoadStats loadStats) {
+  public void setStatistics(RegionLoadStats loadStats) {
     this.stats = loadStats;
   }
 
@@ -1055,8 +987,7 @@ public class Result implements CellScannable, CellScanner {
    * @return the associated statistics about the region from which this was returned. Can be
    * <tt>null</tt> if stats are disabled.
    */
-  @Deprecated
-  public ClientProtos.RegionLoadStats getStats() {
+  public RegionLoadStats getStats() {
     return stats;
   }
 
