@@ -19,8 +19,7 @@
 
 package org.apache.hadoop.hbase.client;
 
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
-import org.apache.hadoop.hbase.classification.InterfaceStability;
+import org.apache.yetus.audience.InterfaceAudience;
 
 /**
  * Select server type i.e destination for RPC request associated with ring buffer.
@@ -28,7 +27,6 @@ import org.apache.hadoop.hbase.classification.InterfaceStability;
  * are maintained by HMaster.
  */
 @InterfaceAudience.Public
-@InterfaceStability.Stable
 public enum ServerType {
   MASTER,
   REGION_SERVER
