@@ -19,7 +19,12 @@
 package org.apache.hadoop.hbase;
 
 import static org.junit.Assert.assertArrayEquals;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -31,23 +36,23 @@ import java.util.List;
 import java.util.Set;
 import java.util.TreeSet;
 
-import junit.framework.TestCase;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.hbase.KeyValue.KVComparator;
-import org.apache.hadoop.hbase.KeyValue.MetaComparator;
-import org.apache.hadoop.hbase.KeyValue.Type;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.ByteBufferUtils;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 @Category(SmallTests.class)
-public class TestKeyValue extends TestCase {
-  private static final Log LOG = LogFactory.getLog(TestKeyValue.class);
+public class TestKeyValue {
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+    HBaseClassTestRule.forClass(TestKeyValue.class);
+  private static final Logger LOG = LoggerFactory.getLogger(TestKeyValue.class);
 
+  @Test
   public void testColumnCompare() {
     final byte[] a = Bytes.toBytes("aaa");
     byte[] family1 = Bytes.toBytes("abc");
@@ -70,6 +75,7 @@ public class TestKeyValue extends TestCase {
   /**
    * Test a corner case when the family qualifier is a prefix of the column qualifier.
    */
+  @Test
   public void testColumnCompare_prefix() {
     final byte[] a = Bytes.toBytes("aaa");
     byte[] family1 = Bytes.toBytes("abc");
@@ -81,14 +87,16 @@ public class TestKeyValue extends TestCase {
     assertFalse(CellUtil.matchingColumn(aaa, family2, qualifier2));
   }
 
+  @Test
   public void testBasics() {
     LOG.info("LOWKEY: " + KeyValue.LOWESTKEY.toString());
-    check(Bytes.toBytes(getName()),
-      Bytes.toBytes(getName()), Bytes.toBytes(getName()), 1,
-      Bytes.toBytes(getName()));
+    String name = "testBasics";
+    check(Bytes.toBytes(name),
+      Bytes.toBytes(name), Bytes.toBytes(name), 1,
+      Bytes.toBytes(name));
     // Test empty value and empty column -- both should work. (not empty fam)
-    check(Bytes.toBytes(getName()), Bytes.toBytes(getName()), null, 1, null);
-    check(HConstants.EMPTY_BYTE_ARRAY, Bytes.toBytes(getName()), null, 1, null);
+    check(Bytes.toBytes(name), Bytes.toBytes(name), null, 1, null);
+    check(HConstants.EMPTY_BYTE_ARRAY, Bytes.toBytes(name), null, 1, null);
     // empty qual is equivalent to null qual
     assertEquals(
       new KeyValue(Bytes.toBytes("rk"), Bytes.toBytes("fam"), null, 1, (byte[]) null),
@@ -106,6 +114,7 @@ public class TestKeyValue extends TestCase {
     LOG.info(kv.toString());
   }
 
+  @Test
   public void testPlainCompare() {
     final byte[] a = Bytes.toBytes("aaa");
     final byte[] b = Bytes.toBytes("bbb");
@@ -113,26 +122,27 @@ public class TestKeyValue extends TestCase {
     final byte[] qf = Bytes.toBytes("umn");
     KeyValue aaa = new KeyValue(a, fam, qf, a);
     KeyValue bbb = new KeyValue(b, fam, qf, b);
-    assertTrue(KeyValue.COMPARATOR.compare(aaa, bbb) < 0);
-    assertTrue(KeyValue.COMPARATOR.compare(bbb, aaa) > 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(aaa, bbb) < 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(bbb, aaa) > 0);
     // Compare breaks if passed same ByteBuffer as both left and right arguments.
-    assertTrue(KeyValue.COMPARATOR.compare(bbb, bbb) == 0);
-    assertTrue(KeyValue.COMPARATOR.compare(aaa, aaa) == 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(bbb, bbb) == 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(aaa, aaa) == 0);
     // Do compare with different timestamps.
     aaa = new KeyValue(a, fam, qf, 1, a);
     bbb = new KeyValue(a, fam, qf, 2, a);
-    assertTrue(KeyValue.COMPARATOR.compare(aaa, bbb) > 0);
-    assertTrue(KeyValue.COMPARATOR.compare(bbb, aaa) < 0);
-    assertTrue(KeyValue.COMPARATOR.compare(aaa, aaa) == 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(aaa, bbb) > 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(bbb, aaa) < 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(aaa, aaa) == 0);
     // Do compare with different types.  Higher numbered types -- Delete
     // should sort ahead of lower numbers; i.e. Put
     aaa = new KeyValue(a, fam, qf, 1, KeyValue.Type.Delete, a);
     bbb = new KeyValue(a, fam, qf, 1, a);
-    assertTrue(KeyValue.COMPARATOR.compare(aaa, bbb) < 0);
-    assertTrue(KeyValue.COMPARATOR.compare(bbb, aaa) > 0);
-    assertTrue(KeyValue.COMPARATOR.compare(aaa, aaa) == 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(aaa, bbb) < 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(bbb, aaa) > 0);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(aaa, aaa) == 0);
   }
 
+  @Test
   public void testMoreComparisons() {
     long now = System.currentTimeMillis();
 
@@ -141,7 +151,7 @@ public class TestKeyValue extends TestCase {
         Bytes.toBytes("TestScanMultipleVersions,row_0500,1236020145502"), now);
     KeyValue bbb = new KeyValue(
         Bytes.toBytes("TestScanMultipleVersions,,99999999999999"), now);
-    KVComparator c = new KeyValue.MetaComparator();
+    CellComparator c = MetaCellComparator.META_COMPARATOR;
     assertTrue(c.compare(bbb, aaa) < 0);
 
     KeyValue aaaa = new KeyValue(Bytes.toBytes("TestScanMultipleVersions,,1236023996656"),
@@ -156,13 +166,14 @@ public class TestKeyValue extends TestCase {
         Bytes.toBytes("info"), Bytes.toBytes("regioninfo"), 1236034574912L,
         (byte[])null);
     assertTrue(c.compare(x, y) < 0);
-    comparisons(new KeyValue.MetaComparator());
-    comparisons(new KeyValue.KVComparator());
-    metacomparisons(new KeyValue.MetaComparator());
+    comparisons(MetaCellComparator.META_COMPARATOR);
+    comparisons(CellComparatorImpl.COMPARATOR);
+    metacomparisons(MetaCellComparator.META_COMPARATOR);
   }
 
+  @Test
   public void testMetaComparatorTableKeysWithCommaOk() {
-    MetaComparator c = new KeyValue.MetaComparator();
+    CellComparator c = MetaCellComparator.META_COMPARATOR;
     long now = System.currentTimeMillis();
     // meta keys values are not quite right.  A users can enter illegal values
     // from shell when scanning meta.
@@ -175,6 +186,7 @@ public class TestKeyValue extends TestCase {
    * Tests cases where rows keys have characters below the ','.
    * See HBASE-832
    */
+  @Test
   public void testKeyValueBorderCases() {
     // % sorts before , so if we don't do special comparator, rowB would
     // come before rowA.
@@ -182,16 +194,16 @@ public class TestKeyValue extends TestCase {
       Bytes.toBytes("fam"), Bytes.toBytes(""), Long.MAX_VALUE, (byte[])null);
     KeyValue rowB = new KeyValue(Bytes.toBytes("testtable,www.hbase.org/%20,99999"),
         Bytes.toBytes("fam"), Bytes.toBytes(""), Long.MAX_VALUE, (byte[])null);
-    assertTrue(KeyValue.META_COMPARATOR.compare(rowA, rowB) < 0);
+    assertTrue(MetaCellComparator.META_COMPARATOR.compare(rowA, rowB) < 0);
 
     rowA = new KeyValue(Bytes.toBytes("testtable,,1234"), Bytes.toBytes("fam"),
         Bytes.toBytes(""), Long.MAX_VALUE, (byte[])null);
     rowB = new KeyValue(Bytes.toBytes("testtable,$www.hbase.org/,99999"),
         Bytes.toBytes("fam"), Bytes.toBytes(""), Long.MAX_VALUE, (byte[])null);
-    assertTrue(KeyValue.META_COMPARATOR.compare(rowA, rowB) < 0);
+    assertTrue(MetaCellComparator.META_COMPARATOR.compare(rowA, rowB) < 0);
   }
 
-  private void metacomparisons(final KeyValue.MetaComparator c) {
+  private void metacomparisons(final CellComparatorImpl c) {
     long now = System.currentTimeMillis();
     assertTrue(c.compare(new KeyValue(
         Bytes.toBytes(TableName.META_TABLE_NAME.getNameAsString()+",a,,0,1"), now),
@@ -208,7 +220,7 @@ public class TestKeyValue extends TestCase {
           Bytes.toBytes(TableName.META_TABLE_NAME.getNameAsString()+",a,,0,1"), now)) > 0);
   }
 
-  private void comparisons(final KeyValue.KVComparator c) {
+  private void comparisons(final CellComparatorImpl c) {
     long now = System.currentTimeMillis();
     assertTrue(c.compare(new KeyValue(
         Bytes.toBytes(TableName.META_TABLE_NAME.getNameAsString()+",,1"), now),
@@ -224,8 +236,9 @@ public class TestKeyValue extends TestCase {
           Bytes.toBytes(TableName.META_TABLE_NAME.getNameAsString()+",,1"), now)) > 0);
   }
 
+  @Test
   public void testBinaryKeys() {
-    Set<KeyValue> set = new TreeSet<>(KeyValue.COMPARATOR);
+    Set<KeyValue> set = new TreeSet<>(CellComparatorImpl.COMPARATOR);
     final byte[] fam = Bytes.toBytes("col");
     final byte[] qf = Bytes.toBytes("umn");
     final byte[] nb = new byte[0];
@@ -245,13 +258,13 @@ public class TestKeyValue extends TestCase {
       for (KeyValue k : set) {
         assertEquals(count++, k.getTimestamp());
       }
-    } catch (junit.framework.AssertionFailedError e) {
+    } catch (java.lang.AssertionError e) {
       // Expected
       assertion = true;
     }
     assertTrue(assertion);
     // Make set with good comparator
-    set = new TreeSet<>(new KeyValue.MetaComparator());
+    set = new TreeSet<>(MetaCellComparator.META_COMPARATOR);
     Collections.addAll(set, keys);
     count = 0;
     for (KeyValue k : set) {
@@ -259,29 +272,36 @@ public class TestKeyValue extends TestCase {
     }
   }
 
+  @Test
+  public void testStackedUpKeyValue() {
+    // Test multiple KeyValues in a single blob.
+
+    // TODO actually write this test!
+  }
+
   private final byte[] rowA = Bytes.toBytes("rowA");
   private final byte[] rowB = Bytes.toBytes("rowB");
 
   private final byte[] family = Bytes.toBytes("family");
   private final byte[] qualA = Bytes.toBytes("qfA");
 
-  private void assertKVLess(KeyValue.KVComparator c, KeyValue less, KeyValue greater) {
+  private void assertKVLess(CellComparator c, KeyValue less, KeyValue greater) {
     int cmp = c.compare(less,greater);
     assertTrue(cmp < 0);
     cmp = c.compare(greater,less);
     assertTrue(cmp > 0);
   }
 
-  private void assertKVLessWithoutRow(KeyValue.KVComparator c, int common, KeyValue less,
-      KeyValue greater) {
+  private void assertKVLessWithoutRow(CellComparator c, KeyValue less, KeyValue greater) {
     int cmp = c.compare(less, greater);
     assertTrue(cmp < 0);
     cmp = c.compare(greater, less);
     assertTrue(cmp > 0);
   }
 
+  @Test
   public void testCompareWithoutRow() {
-    final KVComparator c = new KeyValue.KVComparator();
+    final CellComparator c = CellComparatorImpl.COMPARATOR;
     byte[] row = Bytes.toBytes("row");
 
     byte[] fa = Bytes.toBytes("fa");
@@ -305,9 +325,9 @@ public class TestKeyValue extends TestCase {
     KeyValue kv1_0 = new KeyValue(row, fami1, qual0, ts, KeyValue.Type.Put);
 
     // 'fami:qf1' < 'fami:qf2'
-    assertKVLessWithoutRow(c, 0, kv0_1, kv0_2);
+    assertKVLessWithoutRow(c, kv0_1, kv0_2);
     // 'fami:qf1' < 'fami1:'
-    assertKVLessWithoutRow(c, 0, kv0_1, kv1_0);
+    assertKVLessWithoutRow(c, kv0_1, kv1_0);
 
     // Test comparison by skipping the same prefix bytes.
     /*
@@ -318,17 +338,18 @@ public class TestKeyValue extends TestCase {
     int commonLength = KeyValue.ROW_LENGTH_SIZE + KeyValue.FAMILY_LENGTH_SIZE
         + row.length;
     // 'fa:' < 'fami:'. They have commonPrefix + 2 same prefix bytes.
-    assertKVLessWithoutRow(c, commonLength + 2, kv_0, kv0_0);
+    assertKVLessWithoutRow(c, kv_0, kv0_0);
     // 'fami:' < 'fami:qf1'. They have commonPrefix + 4 same prefix bytes.
-    assertKVLessWithoutRow(c, commonLength + 4, kv0_0, kv0_1);
+    assertKVLessWithoutRow(c, kv0_0, kv0_1);
     // 'fami:qf1' < 'fami1:'. They have commonPrefix + 4 same prefix bytes.
-    assertKVLessWithoutRow(c, commonLength + 4, kv0_1, kv1_0);
+    assertKVLessWithoutRow(c, kv0_1, kv1_0);
     // 'fami:qf1' < 'fami:qf2'. They have commonPrefix + 6 same prefix bytes.
-    assertKVLessWithoutRow(c, commonLength + 6, kv0_1, kv0_2);
+    assertKVLessWithoutRow(c, kv0_1, kv0_2);
   }
 
+  @Test
   public void testFirstLastOnRow() {
-    final KVComparator c = new KeyValue.KVComparator();
+    final CellComparator c = CellComparatorImpl.COMPARATOR;
     long ts = 1;
     byte[] bufferA = new byte[128];
     int offsetA = 0;
@@ -373,6 +394,7 @@ public class TestKeyValue extends TestCase {
     assertKVLess(c, firstOnRowABufferFamQual, lastOnRowA);
   }
 
+  @Test
   public void testCreateKeyOnly() {
     long ts = 1;
     byte[] value = Bytes.toBytes("a real value");
@@ -394,6 +416,7 @@ public class TestKeyValue extends TestCase {
     }
   }
 
+  @Test
   public void testCreateKeyValueFromKey() {
     KeyValue kv = new KeyValue(Bytes.toBytes("myRow"), Bytes.toBytes("myCF"),
         Bytes.toBytes("myQualifier"), 12345L, Bytes.toBytes("myValue"));
@@ -417,6 +440,7 @@ public class TestKeyValue extends TestCase {
    * Tests that getTimestamp() does always return the proper timestamp, even after updating it.
    * See HBASE-6265.
    */
+  @Test
   public void testGetTimestamp() {
     KeyValue kv = new KeyValue(Bytes.toBytes("myRow"), Bytes.toBytes("myCF"),
       Bytes.toBytes("myQualifier"), HConstants.LATEST_TIMESTAMP,
@@ -428,6 +452,7 @@ public class TestKeyValue extends TestCase {
     assertEquals(12345L, time2);
   }
 
+  @Test
   public void testKVsWithTags() {
     byte[] row = Bytes.toBytes("myRow");
     byte[] cf = Bytes.toBytes("myCF");
@@ -436,7 +461,7 @@ public class TestKeyValue extends TestCase {
     byte[] metaValue1 = Bytes.toBytes("metaValue1");
     byte[] metaValue2 = Bytes.toBytes("metaValue2");
     KeyValue kv = new KeyValue(row, cf, q, HConstants.LATEST_TIMESTAMP, value, new Tag[] {
-      new Tag((byte) 1, metaValue1), new Tag((byte) 2, metaValue2)
+      new ArrayBackedTag((byte) 1, metaValue1), new ArrayBackedTag((byte) 2, metaValue2)
     });
     assertTrue(kv.getTagsLength() > 0);
     assertTrue(Bytes.equals(kv.getRowArray(), kv.getRowOffset(), kv.getRowLength(), row, 0,
@@ -447,54 +472,54 @@ public class TestKeyValue extends TestCase {
       kv.getQualifierLength(), q, 0, q.length));
     assertTrue(Bytes.equals(kv.getValueArray(), kv.getValueOffset(), kv.getValueLength(), value, 0,
       value.length));
-    List<Tag> tags = kv.getTags();
+    List<Tag> tags = PrivateCellUtil.getTags(kv);
     assertNotNull(tags);
     assertEquals(2, tags.size());
     boolean meta1Ok = false, meta2Ok = false;
     for (Tag tag : tags) {
       if (tag.getType() == (byte) 1) {
-        if (Bytes.equals(tag.getValue(), metaValue1)) {
+        if (Bytes.equals(Tag.cloneValue(tag), metaValue1)) {
           meta1Ok = true;
         }
       } else {
-        if (Bytes.equals(tag.getValue(), metaValue2)) {
+        if (Bytes.equals(Tag.cloneValue(tag), metaValue2)) {
           meta2Ok = true;
         }
       }
     }
     assertTrue(meta1Ok);
     assertTrue(meta2Ok);
-    Iterator<Tag> tagItr = CellUtil.tagsIterator(kv.getTagsArray(), kv.getTagsOffset(),
-      kv.getTagsLength());
+    Iterator<Tag> tagItr = PrivateCellUtil.tagsIterator(kv);
+
     assertTrue(tagItr.hasNext());
     Tag next = tagItr.next();
-    assertEquals(10, next.getTagLength());
+    assertEquals(10, next.getValueLength());
     assertEquals((byte) 1, next.getType());
-    Bytes.equals(next.getValue(), metaValue1);
+    Bytes.equals(Tag.cloneValue(next), metaValue1);
     assertTrue(tagItr.hasNext());
     next = tagItr.next();
-    assertEquals(10, next.getTagLength());
+    assertEquals(10, next.getValueLength());
     assertEquals((byte) 2, next.getType());
-    Bytes.equals(next.getValue(), metaValue2);
+    Bytes.equals(Tag.cloneValue(next), metaValue2);
     assertFalse(tagItr.hasNext());
 
-    tagItr = CellUtil.tagsIterator(kv.getTagsArray(), kv.getTagsOffset(),
-      kv.getTagsLength());
+    tagItr = PrivateCellUtil.tagsIterator(kv);
     assertTrue(tagItr.hasNext());
     next = tagItr.next();
-    assertEquals(10, next.getTagLength());
+    assertEquals(10, next.getValueLength());
     assertEquals((byte) 1, next.getType());
-    Bytes.equals(next.getValue(), metaValue1);
+    Bytes.equals(Tag.cloneValue(next), metaValue1);
     assertTrue(tagItr.hasNext());
     next = tagItr.next();
-    assertEquals(10, next.getTagLength());
+    assertEquals(10, next.getValueLength());
     assertEquals((byte) 2, next.getType());
-    Bytes.equals(next.getValue(), metaValue2);
+    Bytes.equals(Tag.cloneValue(next), metaValue2);
     assertFalse(tagItr.hasNext());
   }
 
+  @Test
   public void testMetaKeyComparator() {
-    MetaComparator c = new KeyValue.MetaComparator();
+    CellComparator c = MetaCellComparator.META_COMPARATOR;
     long now = System.currentTimeMillis();
 
     KeyValue a = new KeyValue(Bytes.toBytes("table1"), now);
@@ -538,6 +563,7 @@ public class TestKeyValue extends TestCase {
     assertTrue(c.compare(a, b) < 0);
   }
 
+  @Test
   public void testEqualsAndHashCode() {
     KeyValue kvA1 = new KeyValue(Bytes.toBytes("key"), Bytes.toBytes("cf"),
         Bytes.toBytes("qualA"), Bytes.toBytes("1"));
@@ -555,25 +581,27 @@ public class TestKeyValue extends TestCase {
     assertNotEquals(kvA1.hashCode(), kvB.hashCode());
   }
 
+  @Test
   public void testKeyValueSerialization() throws Exception {
     KeyValue[] keyValues = new KeyValue[] {
       new KeyValue(Bytes.toBytes("key"), Bytes.toBytes("cf"), Bytes.toBytes("qualA"),
-        Bytes.toBytes("1")),
+            Bytes.toBytes("1")),
       new KeyValue(Bytes.toBytes("key"), Bytes.toBytes("cf"), Bytes.toBytes("qualA"),
-        Bytes.toBytes("2")),
+            Bytes.toBytes("2")),
       new KeyValue(Bytes.toBytes("key"), Bytes.toBytes("cf"), Bytes.toBytes("qualA"),
-        System.currentTimeMillis(), Bytes.toBytes("2"),
-        new Tag[] {
-          new Tag((byte) 120, "tagA"),
-          new Tag((byte) 121, Bytes.toBytes("tagB")) }),
+            System.currentTimeMillis(), Bytes.toBytes("2"),
+            new Tag[] {
+              new ArrayBackedTag((byte) 120, "tagA"),
+              new ArrayBackedTag((byte) 121, Bytes.toBytes("tagB")) }),
       new KeyValue(Bytes.toBytes("key"), Bytes.toBytes("cf"), Bytes.toBytes("qualA"),
-        System.currentTimeMillis(), Bytes.toBytes("2"),
-        new Tag[] { new Tag((byte) 0, "tagA") }),
+            System.currentTimeMillis(), Bytes.toBytes("2"),
+            new Tag[] { new ArrayBackedTag((byte) 0, "tagA") }),
       new KeyValue(Bytes.toBytes("key"), Bytes.toBytes("cf"), Bytes.toBytes(""),
-        Bytes.toBytes("1")) };
+            Bytes.toBytes("1")) };
     ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
     for (KeyValue kv : keyValues) {
       DataOutputStream os = new DataOutputStream(byteArrayOutputStream);
+      ByteBufferUtils.putInt(os, KeyValueUtil.getSerializedSize(kv, true));
       KeyValueUtil.oswrite(kv, os, true);
     }
     DataInputStream is =
@@ -583,7 +611,7 @@ public class TestKeyValue extends TestCase {
       KeyValue destKv = KeyValueUtil.createKeyValueFromInputStream(is, true);
       assertEquals(keyValues[i], destKv);
       assertArrayEquals(CellUtil.cloneValue(keyValues[i]), CellUtil.cloneValue(destKv));
-      assertArrayEquals(CellUtil.cloneTags(keyValues[i]), CellUtil.cloneTags(destKv));
+      assertArrayEquals(PrivateCellUtil.cloneTags(keyValues[i]), PrivateCellUtil.cloneTags(destKv));
     }
   }
 
@@ -712,23 +740,11 @@ public class TestKeyValue extends TestCase {
         KeyValueUtil.createKeyValueFromInputStream(
           new DataInputStream(new ByteArrayInputStream(baos.toByteArray())), c.withTags);
         if (c.expectedMessage != null) {
-          fail("Should fail when parse key value from an invalid bytes for case#" + i + ". " + c);
+          fail("Should fail when parse kv from an invalid bytes, case#" + i + ". " + c);
         }
       } catch (IllegalArgumentException e) {
         assertEquals("Case#" + i + " failed," + c, c.getExpectedMessage(), e.getMessage());
       }
     }
   }
-
-  @Test
-  public void testRawBytesComparator() {
-    long ts = System.currentTimeMillis();
-    byte[] key = Bytes.toBytes("key");
-    byte[] cf = Bytes.toBytes("cf");
-    byte[] qualifier = Bytes.toBytes("qualifier");
-    byte[] value = Bytes.toBytes("value");
-    KeyValue kvA1 = new KeyValue(key, cf, qualifier, ts, Type.Put, value);
-    KeyValue kvA2 = new KeyValue(key, cf, qualifier, ts, Type.DeleteFamily, value);
-    assertTrue(KeyValue.RAW_COMPARATOR.compare(kvA1, kvA2) > 0);
-  }
 }
