@@ -18,7 +18,6 @@
 package org.apache.hadoop.hbase.util;
 
 import static org.junit.Assert.assertArrayEquals;
-
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.DataInputStream;
@@ -32,16 +31,21 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Random;
-
 import junit.framework.TestCase;
-
-import org.apache.hadoop.hbase.testclassification.SmallTests;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
+import org.apache.hadoop.hbase.testclassification.MediumTests;
+import org.apache.hadoop.hbase.testclassification.MiscTests;
 import org.apache.hadoop.io.WritableUtils;
 import org.junit.Assert;
+import org.junit.ClassRule;
 import org.junit.experimental.categories.Category;
 
-@Category(SmallTests.class)
+@Category({MiscTests.class, MediumTests.class})
 public class TestBytes extends TestCase {
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestBytes.class);
+
   private static void setUnsafe(boolean value) throws Exception {
     Field field = Bytes.class.getDeclaredField("UNSAFE_UNALIGNED");
     field.setAccessible(true);
@@ -331,33 +335,25 @@ public class TestBytes extends TestCase {
     byte[] key4 = { 0 };
     byte[] key5 = { 2 };
 
-    assertEquals(1, Bytes.binarySearch(arr, key1, 0, 1,
-      Bytes.BYTES_RAWCOMPARATOR));
-    assertEquals(0, Bytes.binarySearch(arr, key1, 1, 1,
-      Bytes.BYTES_RAWCOMPARATOR));
+    assertEquals(1, Bytes.binarySearch(arr, key1, 0, 1));
+    assertEquals(0, Bytes.binarySearch(arr, key1, 1, 1));
     assertEquals(-(2+1), Arrays.binarySearch(arr, key2_2,
       Bytes.BYTES_COMPARATOR));
-    assertEquals(-(2+1), Bytes.binarySearch(arr, key2, 0, 1,
-      Bytes.BYTES_RAWCOMPARATOR));
-    assertEquals(4, Bytes.binarySearch(arr, key2, 1, 1,
-      Bytes.BYTES_RAWCOMPARATOR));
-    assertEquals(2, Bytes.binarySearch(arr, key3, 0, 1,
-      Bytes.BYTES_RAWCOMPARATOR));
-    assertEquals(5, Bytes.binarySearch(arr, key3, 1, 1,
-      Bytes.BYTES_RAWCOMPARATOR));
+    assertEquals(-(2+1), Bytes.binarySearch(arr, key2, 0, 1));
+    assertEquals(4, Bytes.binarySearch(arr, key2, 1, 1));
+    assertEquals(2, Bytes.binarySearch(arr, key3, 0, 1));
+    assertEquals(5, Bytes.binarySearch(arr, key3, 1, 1));
     assertEquals(-1,
-      Bytes.binarySearch(arr, key4, 0, 1, Bytes.BYTES_RAWCOMPARATOR));
+      Bytes.binarySearch(arr, key4, 0, 1));
     assertEquals(-2,
-      Bytes.binarySearch(arr, key5, 0, 1, Bytes.BYTES_RAWCOMPARATOR));
+      Bytes.binarySearch(arr, key5, 0, 1));
 
     // Search for values to the left and to the right of each item in the array.
     for (int i = 0; i < arr.length; ++i) {
       assertEquals(-(i + 1), Bytes.binarySearch(arr,
-          new byte[] { (byte) (arr[i][0] - 1) }, 0, 1,
-          Bytes.BYTES_RAWCOMPARATOR));
+          new byte[] { (byte) (arr[i][0] - 1) }, 0, 1));
       assertEquals(-(i + 2), Bytes.binarySearch(arr,
-          new byte[] { (byte) (arr[i][0] + 1) }, 0, 1,
-          Bytes.BYTES_RAWCOMPARATOR));
+          new byte[] { (byte) (arr[i][0] + 1) }, 0, 1));
     }
   }
 
@@ -548,7 +544,7 @@ public class TestBytes extends TestCase {
   }
 
   public void testToFromHex() {
-    List<String> testStrings = new ArrayList<>();
+    List<String> testStrings = new ArrayList<>(8);
     testStrings.addAll(Arrays.asList("", "00", "A0", "ff", "FFffFFFFFFFFFF", "12",
       "0123456789abcdef", "283462839463924623984692834692346ABCDFEDDCA0"));
     for (String testString : testStrings) {
@@ -558,8 +554,9 @@ public class TestBytes extends TestCase {
       Assert.assertTrue(testString.equalsIgnoreCase(result));
     }
 
-    List<byte[]> testByteData = new ArrayList<>(Arrays.asList(new byte[0], new byte[1],
-      new byte[10], new byte[] { 1, 2, 3, 4, 5 }, new byte[] { (byte) 0xFF }));
+    List<byte[]> testByteData = new ArrayList<>(5);
+    testByteData.addAll(Arrays.asList(new byte[0], new byte[1], new byte[10],
+      new byte[] { 1, 2, 3, 4, 5 }, new byte[] { (byte) 0xFF }));
     Random r = new Random();
     for (int i = 0; i < 20; i++) {
       byte[] bytes = new byte[r.nextInt(100)];
@@ -575,3 +572,4 @@ public class TestBytes extends TestCase {
     }
   }
 }
+
