@@ -17,13 +17,9 @@
  */
 package org.apache.hadoop.hbase.hbtop.terminal.impl.batch;
 
-import org.apache.hadoop.hbase.hbtop.terminal.AbstractTerminalPrinter;
 import org.apache.hadoop.hbase.hbtop.terminal.TerminalPrinter;
 
-/**
- * An implementation of the {@link TerminalPrinter} interface for batch mode.
- */
-public class BatchTerminalPrinter extends AbstractTerminalPrinter {
+public class BatchTerminalPrinter implements TerminalPrinter {
 
   @Override
   public TerminalPrinter print(String value) {
