@@ -22,23 +22,30 @@ import java.util.ArrayList;
 import java.util.Comparator;
 import java.util.List;
 import java.util.Locale;
+import java.util.Optional;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicInteger;
-import java.util.concurrent.atomic.AtomicLong;
+import java.util.concurrent.atomic.LongAdder;
+import java.util.Map;
+import java.util.HashMap;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.Abortable;
 import org.apache.hadoop.hbase.HConstants;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
+import org.apache.hadoop.hbase.conf.ConfigurationObserver;
+import org.apache.hbase.thirdparty.io.netty.util.internal.StringUtil;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.apache.hadoop.hbase.monitoring.MonitoredRPCHandler;
 import org.apache.hadoop.hbase.util.BoundedPriorityBlockingQueue;
 import org.apache.hadoop.hbase.util.ReflectionUtils;
+import org.apache.hadoop.util.StringUtils;
 
-import com.google.common.base.Preconditions;
-import com.google.common.base.Strings;
+import org.apache.hbase.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hbase.thirdparty.com.google.common.base.Strings;
 
 /**
  * Runs the CallRunners passed here via {@link #dispatch(CallRunner)}. Subclass and add particular
@@ -46,7 +53,7 @@ import com.google.common.base.Strings;
  */
 @InterfaceAudience.Private
 public abstract class RpcExecutor {
-  private static final Log LOG = LogFactory.getLog(RpcExecutor.class);
+  private static final Logger LOG = LoggerFactory.getLogger(RpcExecutor.class);
 
   protected static final int DEFAULT_CALL_QUEUE_SIZE_HARD_LIMIT = 250;
   public static final String CALL_QUEUE_HANDLER_FACTOR_CONF_KEY = "hbase.ipc.server.callqueue.handler.factor";
@@ -62,6 +69,7 @@ public abstract class RpcExecutor {
   public static final String CALL_QUEUE_TYPE_CODEL_CONF_VALUE = "codel";
   public static final String CALL_QUEUE_TYPE_DEADLINE_CONF_VALUE = "deadline";
   public static final String CALL_QUEUE_TYPE_FIFO_CONF_VALUE = "fifo";
+  public static final String CALL_QUEUE_TYPE_PLUGGABLE_CONF_VALUE = "pluggable";
   public static final String CALL_QUEUE_TYPE_CONF_KEY = "hbase.ipc.server.callqueue.type";
   public static final String CALL_QUEUE_TYPE_CONF_DEFAULT = CALL_QUEUE_TYPE_FIFO_CONF_VALUE;
 
@@ -74,8 +82,13 @@ public abstract class RpcExecutor {
   public static final int CALL_QUEUE_CODEL_DEFAULT_INTERVAL = 100;
   public static final double CALL_QUEUE_CODEL_DEFAULT_LIFO_THRESHOLD = 0.8;
 
-  private AtomicLong numGeneralCallsDropped = new AtomicLong();
-  private AtomicLong numLifoModeSwitches = new AtomicLong();
+  public static final String PLUGGABLE_CALL_QUEUE_CLASS_NAME =
+    "hbase.ipc.server.callqueue.pluggable.queue.class.name";
+  public static final String PLUGGABLE_CALL_QUEUE_WITH_FAST_PATH_ENABLED =
+    "hbase.ipc.server.callqueue.pluggable.queue.fast.path.enabled";
+
+  private LongAdder numGeneralCallsDropped = new LongAdder();
+  private LongAdder numLifoModeSwitches = new LongAdder();
 
   protected final int numCallQueues;
   protected final List<BlockingQueue<CallRunner>> queues;
@@ -87,35 +100,16 @@ public abstract class RpcExecutor {
   protected volatile int currentQueueLimit;
 
   private final AtomicInteger activeHandlerCount = new AtomicInteger(0);
-  private final List<RpcHandler> handlers;
+  private final List<Handler> handlers;
   private final int handlerCount;
   private final AtomicInteger failedHandlerCount = new AtomicInteger(0);
 
   private String name;
+  private boolean running;
 
   private Configuration conf = null;
   private Abortable abortable = null;
 
-  @Deprecated
-  public RpcExecutor(final String name, final int handlerCount, final int numCallQueues) {
-    this.name = Strings.nullToEmpty(name);
-    this.handlers = new ArrayList<RpcHandler>(handlerCount);
-    this.handlerCount = handlerCount;
-    this.numCallQueues = numCallQueues;
-    this.queues = new ArrayList<>(this.numCallQueues);
-    this.queueClass = null;
-    this.queueInitArgs = new Object[0];
-    this.priority = null;
-  }
-
-  @Deprecated
-  public RpcExecutor(final String name, final int handlerCount, final int numCallQueues,
-      final Configuration conf, final Abortable abortable) {
-    this(name, handlerCount, numCallQueues);
-    this.conf = conf;
-    this.abortable = abortable;
-  }
-
   public RpcExecutor(final String name, final int handlerCount, final int maxQueueLength,
       final PriorityFunction priority, final Configuration conf, final Abortable abortable) {
     this(name, handlerCount, conf.get(CALL_QUEUE_TYPE_CONF_KEY,
@@ -129,7 +123,7 @@ public abstract class RpcExecutor {
     this.conf = conf;
     this.abortable = abortable;
 
-    float callQueuesHandlersFactor = this.conf.getFloat(CALL_QUEUE_HANDLER_FACTOR_CONF_KEY, 0);
+    float callQueuesHandlersFactor = this.conf.getFloat(CALL_QUEUE_HANDLER_FACTOR_CONF_KEY, 0.1f);
     if (Float.compare(callQueuesHandlersFactor, 1.0f) > 0 ||
         Float.compare(0.0f, callQueuesHandlersFactor) > 0) {
       LOG.warn(CALL_QUEUE_HANDLER_FACTOR_CONF_KEY +
@@ -164,42 +158,96 @@ public abstract class RpcExecutor {
       int codelInterval = conf.getInt(CALL_QUEUE_CODEL_INTERVAL, CALL_QUEUE_CODEL_DEFAULT_INTERVAL);
       double codelLifoThreshold = conf.getDouble(CALL_QUEUE_CODEL_LIFO_THRESHOLD,
         CALL_QUEUE_CODEL_DEFAULT_LIFO_THRESHOLD);
-      queueInitArgs = new Object[] { maxQueueLength, codelTargetDelay, codelInterval,
+      this.queueInitArgs = new Object[] { maxQueueLength, codelTargetDelay, codelInterval,
           codelLifoThreshold, numGeneralCallsDropped, numLifoModeSwitches };
-      queueClass = AdaptiveLifoCoDelCallQueue.class;
+      this.queueClass = AdaptiveLifoCoDelCallQueue.class;
+    } else if (isPluggableQueueType(callQueueType)) {
+      Optional<Class<? extends BlockingQueue<CallRunner>>> pluggableQueueClass = getPluggableQueueClass();
+
+      if (!pluggableQueueClass.isPresent()) {
+        throw new PluggableRpcQueueNotFound("Pluggable call queue failed to load and selected call"
+          + " queue type required");
+      } else {
+        this.queueInitArgs = new Object[] { maxQueueLength, this.priority, conf };
+        this.queueClass = pluggableQueueClass.get();
+      }
     } else {
       this.name += ".Fifo";
-      queueInitArgs = new Object[] { maxQueueLength };
-      queueClass = LinkedBlockingQueue.class;
+      this.queueInitArgs = new Object[] { maxQueueLength };
+      this.queueClass = LinkedBlockingQueue.class;
     }
 
-    LOG.info("RpcExecutor " + " name " + " using " + callQueueType
-        + " as call queue; numCallQueues=" + numCallQueues + "; maxQueueLength=" + maxQueueLength
-        + "; handlerCount=" + handlerCount);
+    LOG.info("Instantiated {} with queueClass={}; " +
+        "numCallQueues={}, maxQueueLength={}, handlerCount={}",
+        this.name, this.queueClass, this.numCallQueues, maxQueueLength, this.handlerCount);
   }
 
   protected int computeNumCallQueues(final int handlerCount, final float callQueuesHandlersFactor) {
-    return Math.max(1, (int) Math.round(handlerCount * callQueuesHandlersFactor));
+    return Math.max(1, Math.round(handlerCount * callQueuesHandlersFactor));
   }
 
+  public Map<String, Long> getCallQueueCountsSummary() {
+    HashMap<String, Long> callQueueMethodTotalCount = new HashMap<>();
+
+    for(BlockingQueue<CallRunner> queue: queues) {
+      for (CallRunner cr:queue) {
+        RpcCall rpcCall = cr.getRpcCall();
+
+        String method;
+
+        if (null==rpcCall.getMethod() ||
+             StringUtil.isNullOrEmpty(method = rpcCall.getMethod().getName())) {
+          method = "Unknown";
+        }
+
+        callQueueMethodTotalCount.put(method, 1+callQueueMethodTotalCount.getOrDefault(method, 0L));
+      }
+    }
+
+    return callQueueMethodTotalCount;
+  }
+
+  public Map<String, Long> getCallQueueSizeSummary() {
+    HashMap<String, Long> callQueueMethodTotalSize = new HashMap<>();
+
+    for(BlockingQueue<CallRunner> queue: queues) {
+      for (CallRunner cr:queue) {
+        RpcCall rpcCall = cr.getRpcCall();
+        String method;
+
+        if (null==rpcCall.getMethod() ||
+          StringUtil.isNullOrEmpty(method = rpcCall.getMethod().getName())) {
+          method = "Unknown";
+        }
+
+        long size = rpcCall.getSize();
+
+        callQueueMethodTotalSize.put(method, size+callQueueMethodTotalSize.getOrDefault(method, 0L));
+      }
+    }
+
+    return callQueueMethodTotalSize;
+  }
+
+
   protected void initializeQueues(final int numQueues) {
     if (queueInitArgs.length > 0) {
       currentQueueLimit = (int) queueInitArgs[0];
       queueInitArgs[0] = Math.max((int) queueInitArgs[0], DEFAULT_CALL_QUEUE_SIZE_HARD_LIMIT);
     }
     for (int i = 0; i < numQueues; ++i) {
-      queues
-          .add((BlockingQueue<CallRunner>) ReflectionUtils.newInstance(queueClass, queueInitArgs));
+      queues.add(ReflectionUtils.newInstance(queueClass, queueInitArgs));
     }
   }
 
   public void start(final int port) {
+    running = true;
     startHandlers(port);
   }
 
   public void stop() {
-    for (RpcHandler handler : handlers) {
-      handler.stopRunning();
+    running = false;
+    for (Thread handler : handlers) {
       handler.interrupt();
     }
   }
@@ -208,7 +256,7 @@ public abstract class RpcExecutor {
   public abstract boolean dispatch(final CallRunner callTask) throws InterruptedException;
 
   /** Returns the list of request queues */
-  public List<BlockingQueue<CallRunner>> getQueues() {
+  protected List<BlockingQueue<CallRunner>> getQueues() {
     return queues;
   }
 
@@ -220,12 +268,9 @@ public abstract class RpcExecutor {
   /**
    * Override if providing alternate Handler implementation.
    */
-  protected RpcHandler getHandler(final String name, final double handlerFailureThreshhold,
-      final int handlerCount, final BlockingQueue<CallRunner> q,
-      final AtomicInteger activeHandlerCount, final AtomicInteger failedHandlerCount,
-      final Abortable abortable) {
-    return new RpcHandler(name, handlerFailureThreshhold, handlerCount, q, activeHandlerCount,
-      failedHandlerCount, abortable);
+  protected Handler getHandler(final String name, final double handlerFailureThreshhold,
+      final BlockingQueue<CallRunner> q, final AtomicInteger activeHandlerCount) {
+    return new Handler(name, handlerFailureThreshhold, q, activeHandlerCount);
   }
 
   /**
@@ -242,12 +287,97 @@ public abstract class RpcExecutor {
       final int index = qindex + (i % qsize);
       String name = "RpcServer." + threadPrefix + ".handler=" + handlers.size() + ",queue=" + index
           + ",port=" + port;
-      RpcHandler handler = getHandler(name, handlerFailureThreshhold, handlerCount,
-        callQueues.get(index), activeHandlerCount, failedHandlerCount, abortable);
+      Handler handler = getHandler(name, handlerFailureThreshhold, callQueues.get(index),
+        activeHandlerCount);
       handler.start();
-      LOG.debug("Started " + name);
       handlers.add(handler);
     }
+    LOG.debug("Started handlerCount={} with threadPrefix={}, numCallQueues={}, port={}",
+        handlers.size(), threadPrefix, qsize, port);
+  }
+
+  /**
+   * Handler thread run the {@link CallRunner#run()} in.
+   */
+  protected class Handler extends Thread {
+    /**
+     * Q to find CallRunners to run in.
+     */
+    final BlockingQueue<CallRunner> q;
+
+    final double handlerFailureThreshhold;
+
+    // metrics (shared with other handlers)
+    final AtomicInteger activeHandlerCount;
+
+    Handler(final String name, final double handlerFailureThreshhold,
+        final BlockingQueue<CallRunner> q, final AtomicInteger activeHandlerCount) {
+      super(name);
+      setDaemon(true);
+      this.q = q;
+      this.handlerFailureThreshhold = handlerFailureThreshhold;
+      this.activeHandlerCount = activeHandlerCount;
+    }
+
+    /**
+     * @return A {@link CallRunner}
+     * @throws InterruptedException
+     */
+    protected CallRunner getCallRunner() throws InterruptedException {
+      return this.q.take();
+    }
+
+    @Override
+    public void run() {
+      boolean interrupted = false;
+      try {
+        while (running) {
+          try {
+            run(getCallRunner());
+          } catch (InterruptedException e) {
+            interrupted = true;
+          }
+        }
+      } catch (Exception e) {
+        LOG.warn(e.toString(), e);
+        throw e;
+      } finally {
+        if (interrupted) {
+          Thread.currentThread().interrupt();
+        }
+      }
+    }
+
+    private void run(CallRunner cr) {
+      MonitoredRPCHandler status = RpcServer.getStatus();
+      cr.setStatus(status);
+      try {
+        this.activeHandlerCount.incrementAndGet();
+        cr.run();
+      } catch (Throwable e) {
+        if (e instanceof Error) {
+          int failedCount = failedHandlerCount.incrementAndGet();
+          if (this.handlerFailureThreshhold >= 0
+              && failedCount > handlerCount * this.handlerFailureThreshhold) {
+            String message = "Number of failed RpcServer handler runs exceeded threshhold "
+                + this.handlerFailureThreshhold + "; reason: " + StringUtils.stringifyException(e);
+            if (abortable != null) {
+              abortable.abort(message, e);
+            } else {
+              LOG.error("Error but can't abort because abortable is null: "
+                  + StringUtils.stringifyException(e));
+              throw e;
+            }
+          } else {
+            LOG.warn("Handler errors " + StringUtils.stringifyException(e));
+          }
+        } else {
+          LOG.warn("Handler  exception " + StringUtils.stringifyException(e));
+        }
+      } finally {
+        this.activeHandlerCount.decrementAndGet();
+      }
+    }
   }
 
   public static abstract class QueueBalancer {
@@ -286,6 +416,7 @@ public abstract class RpcExecutor {
       this.queueSize = queueSize;
     }
 
+    @Override
     public int getNextQueue() {
       return ThreadLocalRandom.current().nextInt(queueSize);
     }
@@ -310,12 +441,12 @@ public abstract class RpcExecutor {
 
     @Override
     public int compare(CallRunner a, CallRunner b) {
-      RpcServer.Call callA = a.getCall();
-      RpcServer.Call callB = b.getCall();
-      long deadlineA = priority.getDeadline(callA.getHeader(), callA.param);
-      long deadlineB = priority.getDeadline(callB.getHeader(), callB.param);
-      deadlineA = callA.timestamp + Math.min(deadlineA, maxDelay);
-      deadlineB = callB.timestamp + Math.min(deadlineB, maxDelay);
+      RpcCall callA = a.getRpcCall();
+      RpcCall callB = b.getRpcCall();
+      long deadlineA = priority.getDeadline(callA.getHeader(), callA.getParam());
+      long deadlineB = priority.getDeadline(callB.getHeader(), callB.getParam());
+      deadlineA = callA.getReceiveTime() + Math.min(deadlineA, maxDelay);
+      deadlineB = callB.getReceiveTime() + Math.min(deadlineB, maxDelay);
       return Long.compare(deadlineA, deadlineB);
     }
   }
@@ -332,12 +463,46 @@ public abstract class RpcExecutor {
     return callQueueType.equals(CALL_QUEUE_TYPE_FIFO_CONF_VALUE);
   }
 
+  public static boolean isPluggableQueueType(String callQueueType) {
+    return callQueueType.equals(CALL_QUEUE_TYPE_PLUGGABLE_CONF_VALUE);
+  }
+
+  public static boolean isPluggableQueueWithFastPath(String callQueueType, Configuration conf) {
+    return isPluggableQueueType(callQueueType) &&
+      conf.getBoolean(PLUGGABLE_CALL_QUEUE_WITH_FAST_PATH_ENABLED, false);
+  }
+
+  private Optional<Class<? extends BlockingQueue<CallRunner>>> getPluggableQueueClass() {
+    String queueClassName = conf.get(PLUGGABLE_CALL_QUEUE_CLASS_NAME);
+
+    if (queueClassName == null) {
+      LOG.error("Pluggable queue class config at " + PLUGGABLE_CALL_QUEUE_CLASS_NAME +
+        " was not found");
+      return Optional.empty();
+    }
+
+    try {
+      Class<?> clazz = Class.forName(queueClassName);
+
+      if (BlockingQueue.class.isAssignableFrom(clazz)) {
+        return Optional.of((Class<? extends BlockingQueue<CallRunner>>) clazz);
+      } else {
+        LOG.error("Pluggable Queue class " + queueClassName +
+          " does not extend BlockingQueue<CallRunner>");
+        return Optional.empty();
+      }
+    } catch (ClassNotFoundException exception) {
+      LOG.error("Could not find " + queueClassName + " on the classpath to load.");
+      return Optional.empty();
+    }
+  }
+
   public long getNumGeneralCallsDropped() {
-    return numGeneralCallsDropped.get();
+    return numGeneralCallsDropped.longValue();
   }
 
   public long getNumLifoModeSwitches() {
-    return numLifoModeSwitches.get();
+    return numLifoModeSwitches.longValue();
   }
 
   public int getActiveHandlerCount() {
@@ -405,6 +570,8 @@ public abstract class RpcExecutor {
       if (queue instanceof AdaptiveLifoCoDelCallQueue) {
         ((AdaptiveLifoCoDelCallQueue) queue).updateTunables(codelTargetDelay, codelInterval,
           codelLifoThreshold);
+      } else if (queue instanceof ConfigurationObserver) {
+        ((ConfigurationObserver)queue).onConfigurationChange(conf);
       }
     }
   }
