@@ -21,12 +21,10 @@ package org.apache.hadoop.hbase.regionserver;
 
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
@@ -37,28 +35,47 @@ import org.apache.hadoop.hbase.io.HFileLink;
 import org.apache.hadoop.hbase.io.HalfStoreFileReader;
 import org.apache.hadoop.hbase.io.Reference;
 import org.apache.hadoop.hbase.io.hfile.CacheConfig;
+import org.apache.hadoop.hbase.io.hfile.HFileInfo;
+import org.apache.hadoop.hbase.io.hfile.ReaderContext;
+import org.apache.hadoop.hbase.io.hfile.ReaderContext.ReaderType;
+import org.apache.hadoop.hbase.io.hfile.ReaderContextBuilder;
 import org.apache.hadoop.hbase.util.FSUtils;
+import org.apache.hadoop.hbase.util.Pair;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Describe a StoreFile (hfile, reference, link)
  */
 @InterfaceAudience.Private
 public class StoreFileInfo {
-  private static final Log LOG = LogFactory.getLog(StoreFileInfo.class);
+  private static final Logger LOG = LoggerFactory.getLogger(StoreFileInfo.class);
 
   /**
    * A non-capture group, for hfiles, so that this can be embedded.
    * HFiles are uuid ([0-9a-z]+). Bulk loaded hfiles has (_SeqId_[0-9]+_) has suffix.
+   * The mob del file has (_del) as suffix.
    */
-  public static final String HFILE_NAME_REGEX = "[0-9a-f]+(?:_SeqId_[0-9]+_)?";
+  public static final String HFILE_NAME_REGEX = "[0-9a-f]+(?:(?:_SeqId_[0-9]+_)|(?:_del))?";
 
   /** Regex that will work for hfiles */
   private static final Pattern HFILE_NAME_PATTERN =
     Pattern.compile("^(" + HFILE_NAME_REGEX + ")");
 
   /**
-   * Regex that will work for straight reference names (<hfile>.<parentEncRegion>)
-   * and hfilelink reference names (<table>=<region>-<hfile>.<parentEncRegion>)
+   * A non-capture group, for del files, so that this can be embedded.
+   * A del file has (_del) as suffix.
+   */
+  public static final String DELFILE_NAME_REGEX = "[0-9a-f]+(?:_del)";
+
+  /** Regex that will work for del files */
+  private static final Pattern DELFILE_NAME_PATTERN =
+    Pattern.compile("^(" + DELFILE_NAME_REGEX + ")");
+
+  /**
+   * Regex that will work for straight reference names ({@code <hfile>.<parentEncRegion>})
+   * and hfilelink reference names ({@code <table>=<region>-<hfile>.<parentEncRegion>})
    * If reference, then the regex has more than just one group.
    * Group 1, hfile/hfilelink pattern, is this file's id.
    * Group 2 '(.+)' is the reference's parent region name.
@@ -67,8 +84,11 @@ public class StoreFileInfo {
     Pattern.compile(String.format("^(%s|%s)\\.(.+)$",
       HFILE_NAME_REGEX, HFileLink.LINK_NAME_REGEX));
 
+  public static final String STORE_FILE_READER_NO_READAHEAD = "hbase.store.reader.no-readahead";
+  public static final boolean DEFAULT_STORE_FILE_READER_NO_READAHEAD = false;
+
   // Configuration
-  private Configuration conf;
+  private final Configuration conf;
 
   // FileSystem handle
   private final FileSystem fs;
@@ -76,6 +96,8 @@ public class StoreFileInfo {
   // HDFS blocks distribution information
   private HDFSBlocksDistribution hdfsBlocksDistribution = null;
 
+  private HFileInfo hfileInfo;
+
   // If this storefile references another, this is the reference instance.
   private final Reference reference;
 
@@ -89,14 +111,31 @@ public class StoreFileInfo {
   // timestamp on when the file was created, is 0 and ignored for reference or link files
   private long createdTimestamp;
 
+  private long size;
+
+  private final boolean primaryReplica;
+
+  private final boolean noReadahead;
+
+  // Counter that is incremented every time a scanner is created on the
+  // store file. It is decremented when the scan on the store file is
+  // done.
+  final AtomicInteger refCount = new AtomicInteger(0);
+
   /**
    * Create a Store File Info
    * @param conf the {@link Configuration} to use
    * @param fs The current file system to use.
    * @param initialPath The {@link Path} of the file
+   * @param primaryReplica true if this is a store file for primary replica, otherwise false.
    */
-  public StoreFileInfo(final Configuration conf, final FileSystem fs, final Path initialPath)
-      throws IOException {
+  public StoreFileInfo(final Configuration conf, final FileSystem fs, final Path initialPath,
+      final boolean primaryReplica) throws IOException {
+    this(conf, fs, null, initialPath, primaryReplica);
+  }
+
+  private StoreFileInfo(final Configuration conf, final FileSystem fs, final FileStatus fileStatus,
+      final Path initialPath, final boolean primaryReplica) throws IOException {
     assert fs != null;
     assert initialPath != null;
     assert conf != null;
@@ -104,12 +143,15 @@ public class StoreFileInfo {
     this.fs = fs;
     this.conf = conf;
     this.initialPath = initialPath;
+    this.primaryReplica = primaryReplica;
+    this.noReadahead = this.conf.getBoolean(STORE_FILE_READER_NO_READAHEAD,
+        DEFAULT_STORE_FILE_READER_NO_READAHEAD);
     Path p = initialPath;
     if (HFileLink.isHFileLink(p)) {
       // HFileLink
       this.reference = null;
       this.link = HFileLink.buildFromHFileLinkPattern(conf, p);
-      if (LOG.isTraceEnabled()) LOG.trace(p + " is a link");
+      LOG.trace("{} is a link", p);
     } else if (isReference(p)) {
       this.reference = Reference.read(fs, p);
       Path referencePath = getReferredToFile(p);
@@ -120,11 +162,17 @@ public class StoreFileInfo {
         // Reference
         this.link = null;
       }
-      if (LOG.isTraceEnabled()) LOG.trace(p + " is a " + reference.getFileRegion() +
-              " reference to " + referencePath);
+      LOG.trace("{} is a {} reference to {}", p, reference.getFileRegion(), referencePath);
     } else if (isHFile(p)) {
       // HFile
-      this.createdTimestamp = fs.getFileStatus(initialPath).getModificationTime();
+      if (fileStatus != null) {
+        this.createdTimestamp = fileStatus.getModificationTime();
+        this.size = fileStatus.getLen();
+      } else {
+        FileStatus fStatus = fs.getFileStatus(initialPath);
+        this.createdTimestamp = fStatus.getModificationTime();
+        this.size = fStatus.getLen();
+      }
       this.reference = null;
       this.link = null;
     } else {
@@ -140,7 +188,7 @@ public class StoreFileInfo {
    */
   public StoreFileInfo(final Configuration conf, final FileSystem fs, final FileStatus fileStatus)
       throws IOException {
-    this(conf, fs, fileStatus.getPath());
+    this(conf, fs, fileStatus, fileStatus.getPath(), true);
   }
 
   /**
@@ -151,13 +199,7 @@ public class StoreFileInfo {
    */
   public StoreFileInfo(final Configuration conf, final FileSystem fs, final FileStatus fileStatus,
       final HFileLink link) {
-    this.fs = fs;
-    this.conf = conf;
-    // initialPath can be null only if we get a link.
-    this.initialPath = (fileStatus == null) ? null : fileStatus.getPath();
-      // HFileLink
-    this.reference = null;
-    this.link = link;
+    this(conf, fs, fileStatus, null, link);
   }
 
   /**
@@ -169,12 +211,7 @@ public class StoreFileInfo {
    */
   public StoreFileInfo(final Configuration conf, final FileSystem fs, final FileStatus fileStatus,
       final Reference reference) {
-    this.fs = fs;
-    this.conf = conf;
-    this.initialPath = fileStatus.getPath();
-    this.createdTimestamp = fileStatus.getModificationTime();
-    this.reference = reference;
-    this.link = null;
+    this(conf, fs, fileStatus, reference, null);
   }
 
   /**
@@ -189,10 +226,21 @@ public class StoreFileInfo {
       final Reference reference, final HFileLink link) {
     this.fs = fs;
     this.conf = conf;
-    this.initialPath = fileStatus.getPath();
-    this.createdTimestamp = fileStatus.getModificationTime();
+    this.primaryReplica = false;
+    this.initialPath = (fileStatus == null) ? null : fileStatus.getPath();
+    this.createdTimestamp = (fileStatus == null) ? 0 :fileStatus.getModificationTime();
     this.reference = reference;
     this.link = link;
+    this.noReadahead = this.conf.getBoolean(STORE_FILE_READER_NO_READAHEAD,
+        DEFAULT_STORE_FILE_READER_NO_READAHEAD);
+  }
+
+  /**
+   * Size of the Hfile
+   * @return size
+   */
+  public long getSize() {
+    return size;
   }
 
   /**
@@ -231,27 +279,30 @@ public class StoreFileInfo {
     return this.hdfsBlocksDistribution;
   }
 
-  /**
-   * Open a Reader for the StoreFile
-   * @param fs The current file system to use.
-   * @param cacheConf The cache configuration and block cache reference.
-   * @return The StoreFile.Reader for the file
-   */
-  public StoreFile.Reader open(final FileSystem fs,
-      final CacheConfig cacheConf, final boolean canUseDropBehind) throws IOException {
+  StoreFileReader createReader(ReaderContext context, CacheConfig cacheConf)
+      throws IOException {
+    StoreFileReader reader = null;
+    if (this.reference != null) {
+      reader = new HalfStoreFileReader(context, hfileInfo, cacheConf, reference, refCount, conf);
+    } else {
+      reader = new StoreFileReader(context, hfileInfo, cacheConf, refCount, conf);
+    }
+    return reader;
+  }
+
+  ReaderContext createReaderContext(boolean doDropBehind, long readahead, ReaderType type)
+      throws IOException {
     FSDataInputStreamWrapper in;
     FileStatus status;
-
-    final boolean doDropBehind = canUseDropBehind && cacheConf.shouldDropBehindCompaction();
     if (this.link != null) {
       // HFileLink
-      in = new FSDataInputStreamWrapper(fs, this.link, doDropBehind);
+      in = new FSDataInputStreamWrapper(fs, this.link, doDropBehind, readahead);
       status = this.link.getFileStatus(fs);
     } else if (this.reference != null) {
       // HFile Reference
       Path referencePath = getReferredToFile(this.getPath());
       try {
-        in = new FSDataInputStreamWrapper(fs, referencePath, doDropBehind);
+        in = new FSDataInputStreamWrapper(fs, referencePath, doDropBehind, readahead);
       } catch (FileNotFoundException fnfe) {
         // Intercept the exception so can insert more info about the Reference; otherwise
         // exception just complains about some random file -- operator doesn't realize it
@@ -262,31 +313,22 @@ public class StoreFileInfo {
       }
       status = fs.getFileStatus(referencePath);
     } else {
-      in = new FSDataInputStreamWrapper(fs, this.getPath(),
-          doDropBehind);
+      in = new FSDataInputStreamWrapper(fs, this.getPath(), doDropBehind, readahead);
       status = fs.getFileStatus(initialPath);
     }
     long length = status.getLen();
-    hdfsBlocksDistribution = computeHDFSBlocksDistribution(fs);
-
-    StoreFile.Reader reader = null;
-    if (this.coprocessorHost != null) {
-      reader = this.coprocessorHost.preStoreFileReaderOpen(fs, this.getPath(), in, length,
-        cacheConf, reference);
+    ReaderContextBuilder contextBuilder = new ReaderContextBuilder()
+        .withInputStreamWrapper(in)
+        .withFileSize(length)
+        .withPrimaryReplicaReader(this.primaryReplica)
+        .withReaderType(type)
+        .withFileSystem(fs);
+    if (this.reference != null) {
+      contextBuilder.withFilePath(this.getPath());
+    } else {
+      contextBuilder.withFilePath(status.getPath());
     }
-    if (reader == null) {
-      if (this.reference != null) {
-        reader = new HalfStoreFileReader(fs, this.getPath(), in, length, cacheConf, reference,
-          conf);
-      } else {
-        reader = new StoreFile.Reader(fs, status.getPath(), in, length, cacheConf, conf);
-      }
-    }
-    if (this.coprocessorHost != null) {
-      reader = this.coprocessorHost.postStoreFileReaderOpen(fs, this.getPath(), in, length,
-        cacheConf, reference, reader);
-    }
-    return reader;
+    return contextBuilder.build();
   }
 
   /**
@@ -294,7 +336,6 @@ public class StoreFileInfo {
    */
   public HDFSBlocksDistribution computeHDFSBlocksDistribution(final FileSystem fs)
       throws IOException {
-
     // guard against the case where we get the FileStatus from link, but by the time we
     // call compute the file is moved again
     if (this.link != null) {
@@ -386,7 +427,7 @@ public class StoreFileInfo {
   @Override
   public String toString() {
     return this.getPath() +
-      (isReference() ? "-" + getReferredToFile(this.getPath()) + "-" + reference : "");
+      (isReference() ? "->" + getReferredToFile(this.getPath()) + "-" + reference : "");
   }
 
   /**
@@ -402,6 +443,23 @@ public class StoreFileInfo {
     return m.matches() && m.groupCount() > 0;
   }
 
+  /**
+   * @param path Path to check.
+   * @return True if the path has format of a del file.
+   */
+  public static boolean isDelFile(final Path path) {
+    return isDelFile(path.getName());
+  }
+
+  /**
+   * @param fileName Sting version of path to validate.
+   * @return True if the file name has format of a del file.
+   */
+  public static boolean isDelFile(final String fileName) {
+    Matcher m = DELFILE_NAME_PATTERN.matcher(fileName);
+    return m.matches() && m.groupCount() > 0;
+  }
+
   /**
    * @param path Path to check.
    * @return True if the path has format of a HStoreFile reference.
@@ -436,7 +494,7 @@ public class StoreFileInfo {
   public static Path getReferredToFile(final Path p) {
     Matcher m = REF_NAME_PATTERN.matcher(p.getName());
     if (m == null || !m.matches()) {
-      LOG.warn("Failed match of store file name " + p.toString());
+      LOG.warn("Failed match of store file name {}", p.toString());
       throw new IllegalArgumentException("Failed match of store file name " +
           p.toString());
     }
@@ -446,10 +504,7 @@ public class StoreFileInfo {
     // Tabledir is up two directories from where Reference was written.
     Path tableDir = p.getParent().getParent().getParent();
     String nameStrippedOfSuffix = m.group(1);
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("reference '" + p + "' to region=" + otherRegion
-        + " hfile=" + nameStrippedOfSuffix);
-    }
+    LOG.trace("reference {} to region={} hfile={}", p, otherRegion, nameStrippedOfSuffix);
 
     // Build up new path with the referenced region in place of our current
     // region in the reference path.  Also strip regionname suffix from name.
@@ -457,6 +512,24 @@ public class StoreFileInfo {
       p.getParent().getName()), nameStrippedOfSuffix);
   }
 
+  /*
+   * Return region and file name referred to by a Reference.
+   * @param referenceFile HFile name which is a Reference.
+   * @return Calculated referenced region and file name.
+   * @throws IllegalArgumentException when referenceFile regex fails to match.
+   */
+  public static Pair<String, String> getReferredToRegionAndFile(final String referenceFile) {
+    Matcher m = REF_NAME_PATTERN.matcher(referenceFile);
+    if (m == null || !m.matches()) {
+      LOG.warn("Failed match of store file name {}", referenceFile);
+      throw new IllegalArgumentException("Failed match of store file name " + referenceFile);
+    }
+    String referencedRegion = m.group(2);
+    String referencedFile = m.group(1);
+    LOG.trace("reference {} to region={} file={}", referenceFile, referencedRegion, referencedFile);
+    return new Pair<>(referencedRegion, referencedFile);
+  }
+
   /**
    * Validate the store file name.
    * @param fileName name of the file to validate
@@ -484,7 +557,7 @@ public class StoreFileInfo {
     // after data loss in hdfs for whatever reason (upgrade, etc.): HBASE-646
     // NOTE: that the HFileLink is just a name, so it's an empty file.
     if (!HFileLink.isHFileLink(p) && fileStatus.getLen() <= 0) {
-      LOG.warn("Skipping " + p + " because it is empty. HBASE-646 DATA LOSS?");
+      LOG.warn("Skipping {} because it is empty. HBASE-646 DATA LOSS?", p);
       return false;
     }
 
@@ -525,49 +598,98 @@ public class StoreFileInfo {
   }
 
   @Override
-  public int hashCode() {
-    final int prime = 31;
-    int result = 1;
-    result = prime * result + ((initialPath == null) ? 0 : initialPath.hashCode());
-    result = prime * result + ((link == null) ? 0 : link.hashCode());
-    result = prime * result + ((reference == null) ? 0 : reference.hashCode());
-    return result;
-  }
+  public boolean equals(Object that) {
+    if (this == that) return true;
+    if (that == null) return false;
+
+    if (!(that instanceof StoreFileInfo)) return false;
+
+    StoreFileInfo o = (StoreFileInfo)that;
+    if (initialPath != null && o.initialPath == null) return false;
+    if (initialPath == null && o.initialPath != null) return false;
+    if (initialPath != o.initialPath && initialPath != null
+            && !initialPath.equals(o.initialPath)) return false;
+
+    if (reference != null && o.reference == null) return false;
+    if (reference == null && o.reference != null) return false;
+    if (reference != o.reference && reference != null
+            && !reference.equals(o.reference)) return false;
+
+    if (link != null && o.link == null) return false;
+    if (link == null && o.link != null) return false;
+    if (link != o.link && link != null && !link.equals(o.link)) return false;
+
+    return true;
+  };
+
 
   @Override
-  public boolean equals(Object obj) {
-    if (this == obj) {
-      return true;
-    }
-    if (obj == null) {
-      return false;
-    }
-    if (getClass() != obj.getClass()) {
-      return false;
-    }
-    StoreFileInfo other = (StoreFileInfo) obj;
-    if (initialPath == null) {
-      if (other.initialPath != null) {
-        return false;
-      }
-    } else if (!initialPath.equals(other.initialPath)) {
-      return false;
-    }
-    if (link == null) {
-      if (other.link != null) {
-        return false;
-      }
-    } else if (!link.equals(other.link)) {
-      return false;
-    }
-    if (reference == null) {
-      if (other.reference != null) {
-        return false;
-      }
-    } else if (!reference.equals(other.reference)) {
-      return false;
-    }
-    return true;
+  public int hashCode() {
+    int hash = 17;
+    hash = hash * 31 + ((reference == null) ? 0 : reference.hashCode());
+    hash = hash * 31 + ((initialPath ==  null) ? 0 : initialPath.hashCode());
+    hash = hash * 31 + ((link == null) ? 0 : link.hashCode());
+    return  hash;
   }
 
+  /**
+   * Return the active file name that contains the real data.
+   * <p>
+   * For referenced hfile, we will return the name of the reference file as it will be used to
+   * construct the StoreFileReader. And for linked hfile, we will return the name of the file being
+   * linked.
+   */
+  public String getActiveFileName() {
+    if (reference != null || link == null) {
+      return initialPath.getName();
+    } else {
+      return HFileLink.getReferencedHFileName(initialPath.getName());
+    }
+  }
+
+  FileSystem getFileSystem() {
+    return this.fs;
+  }
+
+  Configuration getConf() {
+    return this.conf;
+  }
+
+  boolean isNoReadahead() {
+    return this.noReadahead;
+  }
+
+  HFileInfo getHFileInfo() {
+    return hfileInfo;
+  }
+
+  void initHDFSBlocksDistribution() throws IOException {
+    hdfsBlocksDistribution = computeHDFSBlocksDistribution(fs);
+  }
+
+  StoreFileReader preStoreFileReaderOpen(ReaderContext context, CacheConfig cacheConf)
+      throws IOException {
+    StoreFileReader reader = null;
+    if (this.coprocessorHost != null) {
+      reader = this.coprocessorHost.preStoreFileReaderOpen(fs, this.getPath(),
+          context.getInputStreamWrapper(), context.getFileSize(),
+          cacheConf, reference);
+    }
+    return reader;
+  }
+
+  StoreFileReader postStoreFileReaderOpen(ReaderContext context, CacheConfig cacheConf,
+      StoreFileReader reader) throws IOException {
+    StoreFileReader res = reader;
+    if (this.coprocessorHost != null) {
+      res = this.coprocessorHost.postStoreFileReaderOpen(fs, this.getPath(),
+          context.getInputStreamWrapper(), context.getFileSize(),
+          cacheConf, reference, reader);
+    }
+    return res;
+  }
+
+  public void initHFileInfo(ReaderContext context) throws IOException {
+    this.hfileInfo = new HFileInfo(context, conf);
+  }
 }
