@@ -16,59 +16,59 @@
 
 package org.apache.hadoop.hbase.util.compaction;
 
-import com.google.common.base.Joiner;
-import com.google.common.base.Optional;
-import com.google.common.base.Predicate;
-import com.google.common.base.Splitter;
-import com.google.common.collect.Iterables;
-import com.google.common.collect.Lists;
-import com.google.common.collect.Maps;
-import com.google.common.collect.Sets;
 import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
-import org.apache.commons.cli.BasicParser;
-import org.apache.commons.cli.CommandLine;
-import org.apache.commons.cli.CommandLineParser;
-import org.apache.commons.cli.HelpFormatter;
-import org.apache.commons.cli.Option;
-import org.apache.commons.cli.Options;
-import org.apache.commons.cli.ParseException;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.conf.Configured;
 import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.HBaseInterfaceAudience;
-import org.apache.hadoop.hbase.HColumnDescriptor;
 import org.apache.hadoop.hbase.HConstants;
-import org.apache.hadoop.hbase.HRegionInfo;
 import org.apache.hadoop.hbase.HRegionLocation;
 import org.apache.hadoop.hbase.NotServingRegionException;
 import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
 import org.apache.hadoop.hbase.client.Admin;
+import org.apache.hadoop.hbase.client.CompactionState;
 import org.apache.hadoop.hbase.client.Connection;
 import org.apache.hadoop.hbase.client.ConnectionFactory;
-import org.apache.hadoop.hbase.protobuf.generated.AdminProtos;
+import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.util.Tool;
 import org.apache.hadoop.util.ToolRunner;
+import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hbase.thirdparty.com.google.common.base.Joiner;
+import org.apache.hbase.thirdparty.com.google.common.base.Splitter;
+import org.apache.hbase.thirdparty.com.google.common.collect.Iterables;
+import org.apache.hbase.thirdparty.com.google.common.collect.Lists;
+import org.apache.hbase.thirdparty.com.google.common.collect.Maps;
+import org.apache.hbase.thirdparty.com.google.common.collect.Sets;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.CommandLine;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.CommandLineParser;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.DefaultParser;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.HelpFormatter;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.Option;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.Options;
+import org.apache.hbase.thirdparty.org.apache.commons.cli.ParseException;
+
 @InterfaceAudience.LimitedPrivate(HBaseInterfaceAudience.TOOLS)
 public class MajorCompactor extends Configured implements Tool {
 
   private static final Logger LOG = LoggerFactory.getLogger(MajorCompactor.class);
-  static final Set<MajorCompactionRequest> ERRORS = Sets.newHashSet();
+  protected static final Set<MajorCompactionRequest> ERRORS = ConcurrentHashMap.newKeySet();
 
   protected ClusterCompactionQueues clusterCompactionQueues;
   private long timestamp;
@@ -134,11 +134,7 @@ public class MajorCompactor extends Configured implements Tool {
   }
 
   private boolean futuresComplete(List<Future<?>> futures) {
-    Iterables.removeIf(futures, new Predicate<Future<?>>() {
-      @Override public boolean apply(Future<?> input) {
-        return input.isDone();
-      }
-    });
+    futures.removeIf(Future::isDone);
     return futures.isEmpty();
   }
 
@@ -162,23 +158,21 @@ public class MajorCompactor extends Configured implements Tool {
   @InterfaceAudience.Private
   void initializeWorkQueues() throws IOException {
     if (storesToCompact.isEmpty()) {
-      for (HColumnDescriptor a : connection.getTable(tableName).getTableDescriptor()
-          .getFamilies()) {
-        storesToCompact.add(Bytes.toString(a.getName()));
-      }
+      connection.getTable(tableName).getDescriptor().getColumnFamilyNames()
+          .forEach(a -> storesToCompact.add(Bytes.toString(a)));
       LOG.info("No family specified, will execute for all families");
     }
     LOG.info(
         "Initializing compaction queues for table:  " + tableName + " with cf: " + storesToCompact);
 
-    Map<ServerName, List<HRegionInfo>> snRegionMap = getServerRegionsMap();
+    Map<ServerName, List<RegionInfo>> snRegionMap = getServerRegionsMap();
     /*
      * If numservers is specified, stop inspecting regions beyond the numservers, it will serve
      * to throttle and won't end up scanning all the regions in the event there are not many
      * regions to compact based on the criteria.
      */
     for (ServerName sn : getServersToCompact(snRegionMap.keySet())) {
-      List<HRegionInfo> regions = snRegionMap.get(sn);
+      List<RegionInfo> regions = snRegionMap.get(sn);
       LOG.debug("Table: " + tableName + " Server: " + sn + " No of regions: " + regions.size());
 
       /*
@@ -187,7 +181,7 @@ public class MajorCompactor extends Configured implements Tool {
        */
       Collections.shuffle(regions);
       int regionsToCompact = numRegions;
-      for (HRegionInfo hri : regions) {
+      for (RegionInfo hri : regions) {
         if (numRegions > 0 && regionsToCompact <= 0) {
           LOG.debug("Reached region limit for server: " + sn);
           break;
@@ -205,10 +199,9 @@ public class MajorCompactor extends Configured implements Tool {
     }
   }
 
-  protected Optional<MajorCompactionRequest> getMajorCompactionRequest(HRegionInfo hri)
+  protected Optional<MajorCompactionRequest> getMajorCompactionRequest(RegionInfo hri)
       throws IOException {
-    return MajorCompactionRequest.newRequest(connection.getConfiguration(), hri, storesToCompact,
-            timestamp);
+    return MajorCompactionRequest.newRequest(connection, hri, storesToCompact, timestamp);
   }
 
   private Collection<ServerName> getServersToCompact(Set<ServerName> snSet) {
@@ -222,15 +215,15 @@ public class MajorCompactor extends Configured implements Tool {
     }
   }
 
-  private Map<ServerName, List<HRegionInfo>> getServerRegionsMap() throws IOException {
-    Map<ServerName, List<HRegionInfo>> snRegionMap = Maps.newHashMap();
+  private Map<ServerName, List<RegionInfo>> getServerRegionsMap() throws IOException {
+    Map<ServerName, List<RegionInfo>> snRegionMap = Maps.newHashMap();
     List<HRegionLocation> regionLocations =
         connection.getRegionLocator(tableName).getAllRegionLocations();
     for (HRegionLocation regionLocation : regionLocations) {
       ServerName sn = regionLocation.getServerName();
-      HRegionInfo hri = regionLocation.getRegionInfo();
+      RegionInfo hri = regionLocation.getRegion();
       if (!snRegionMap.containsKey(sn)) {
-        snRegionMap.put(sn, Lists.<HRegionInfo>newArrayList());
+        snRegionMap.put(sn, Lists.newArrayList());
       }
       snRegionMap.get(sn).add(hri);
     }
@@ -344,11 +337,10 @@ public class MajorCompactor extends Configured implements Tool {
   }
 
   private boolean isCompacting(MajorCompactionRequest request) throws Exception {
-    AdminProtos.GetRegionInfoResponse.CompactionState compactionState = connection.getAdmin()
+    CompactionState compactionState = connection.getAdmin()
         .getCompactionStateForRegion(request.getRegion().getEncodedNameAsBytes());
-    return compactionState.equals(AdminProtos.GetRegionInfoResponse.CompactionState.MAJOR)
-        || compactionState
-        .equals(AdminProtos.GetRegionInfoResponse.CompactionState.MAJOR_AND_MINOR);
+    return compactionState.equals(CompactionState.MAJOR) || compactionState
+        .equals(CompactionState.MAJOR_AND_MINOR);
   }
 
   private void addNewRegions() {
@@ -356,13 +348,11 @@ public class MajorCompactor extends Configured implements Tool {
       List<HRegionLocation> locations =
           connection.getRegionLocator(tableName).getAllRegionLocations();
       for (HRegionLocation location : locations) {
-        if (location.getRegionInfo().getRegionId() > timestamp) {
-          Optional<MajorCompactionRequest> compactionRequest =
-              getMajorCompactionRequest(location.getRegionInfo());
-          if (compactionRequest.isPresent()) {
-            clusterCompactionQueues
-                .addToCompactionQueue(location.getServerName(), compactionRequest.get());
-          }
+        if (location.getRegion().getRegionId() > timestamp) {
+          Optional<MajorCompactionRequest> compactionRequest = MajorCompactionRequest
+              .newRequest(connection, location.getRegion(), storesToCompact, timestamp);
+          compactionRequest.ifPresent(request -> clusterCompactionQueues
+              .addToCompactionQueue(location.getServerName(), request));
         }
       }
     } catch (IOException e) {
@@ -377,57 +367,99 @@ public class MajorCompactor extends Configured implements Tool {
 
   protected Options getCommonOptions() {
     Options options = new Options();
-    Option serverOption = new Option("servers", true, "Concurrent servers compacting");
-    serverOption.setRequired(true);
-    options.addOption(serverOption);
 
-    Option sleepOption = new Option("sleep", true, "Time to sleepForMs (ms) for checking "
-        + "compaction status per region and available "
-        + "work queues: default 30s");
-    options.addOption(sleepOption);
+    options.addOption(
+        Option.builder("servers")
+            .required()
+            .desc("Concurrent servers compacting")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("minModTime").
+            desc("Compact if store files have modification time < minModTime")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("zk")
+            .optionalArg(true)
+            .desc("zk quorum")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("rootDir")
+            .optionalArg(true)
+            .desc("hbase.rootDir")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("sleep")
+            .desc("Time to sleepForMs (ms) for checking compaction status per region and available "
+                + "work queues: default 30s")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("retries")
+        .desc("Max # of retries for a compaction request," + " defaults to 3")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("dryRun")
+            .desc("Dry run, will just output a list of regions that require compaction based on "
+            + "parameters passed")
+            .hasArg(false)
+            .build()
+    );
 
-    Option retryOption = new Option("retries", true, "Max # of retries for a compaction request,"
-        + " defaults to 3");
-    options.addOption(retryOption);
+    options.addOption(
+        Option.builder("skipWait")
+            .desc("Skip waiting after triggering compaction.")
+            .hasArg(false)
+            .build()
+    );
 
-    options.addOption(new Option("dryRun", false, "Dry run, will just output a list of regions"
-        + " that require compaction based on parameters passed"));
+    options.addOption(
+        Option.builder("numservers")
+            .optionalArg(true)
+            .desc("Number of servers to compact in this run, defaults to all")
+            .hasArg()
+            .build()
+    );
 
-    options.addOption(new Option("skipWait", false, "Skip waiting after triggering compaction."));
-
-    Option numServersOption = new Option("numservers", true, "Number of servers to compact in "
-        + "this run, defaults to all");
-    options.addOption(numServersOption);
-
-    Option numRegionsOption = new Option("numregions", true, "Number of regions to compact per"
-        + "server, defaults to all");
-    options.addOption(numRegionsOption);
+    options.addOption(
+        Option.builder("numregions")
+            .optionalArg(true)
+            .desc("Number of regions to compact per server, defaults to all")
+            .hasArg()
+            .build()
+    );
     return options;
   }
 
   @Override
   public int run(String[] args) throws Exception {
     Options options = getCommonOptions();
-    Option tableOption = new Option("table", true, "table name");
-    tableOption.setRequired(true);
-    options.addOption(tableOption);
+    options.addOption(
+        Option.builder("table")
+            .required()
+            .desc("table name")
+            .hasArg()
+            .build()
+    );
+    options.addOption(
+        Option.builder("cf")
+            .optionalArg(true)
+            .desc("column families: comma separated eg: a,b,c")
+            .hasArg()
+            .build()
+    );
 
-    Option cfOption = new Option("cf", true, "column families: comma separated eg: a,b,c");
-    cfOption.setOptionalArg(true);
-    options.addOption(cfOption);
-
-    options.addOption(new Option("minModTime", true,
-        "Compact if store files have modification time < minModTime"));
-
-    Option zkOption = new Option("zk", true, "zk quorum");
-    zkOption.setOptionalArg(true);
-    options.addOption(zkOption);
-
-    Option rootDirOption = new Option("rootDir", true, "hbase.rootDir");
-    rootDirOption.setOptionalArg(true);
-    options.addOption(rootDirOption);
-
-    final CommandLineParser cmdLineParser =  new BasicParser();
+    final CommandLineParser cmdLineParser = new DefaultParser();
     CommandLine commandLine = null;
     try {
       commandLine = cmdLineParser.parse(options, args);
@@ -436,9 +468,13 @@ public class MajorCompactor extends Configured implements Tool {
           "ERROR: Unable to parse command-line arguments " + Arrays.toString(args) + " due to: "
               + parseException);
       printUsage(options);
-      throw parseException;
+      return -1;
+    }
+    if (commandLine == null) {
+      System.out.println("ERROR: Failed parse, empty commandLine; " + Arrays.toString(args));
+      printUsage(options);
+      return -1;
     }
-
     String tableName = commandLine.getOptionValue("table");
     String cf = commandLine.getOptionValue("cf", null);
     Set<String> families = Sets.newHashSet();
@@ -446,14 +482,14 @@ public class MajorCompactor extends Configured implements Tool {
       Iterables.addAll(families, Splitter.on(",").split(cf));
     }
 
-    Configuration configuration = HBaseConfiguration.create();
+    Configuration configuration = getConf();
     int concurrency = Integer.parseInt(commandLine.getOptionValue("servers"));
     long minModTime = Long.parseLong(
         commandLine.getOptionValue("minModTime", String.valueOf(System.currentTimeMillis())));
     String quorum =
         commandLine.getOptionValue("zk", configuration.get(HConstants.ZOOKEEPER_QUORUM));
     String rootDir = commandLine.getOptionValue("rootDir", configuration.get(HConstants.HBASE_DIR));
-    long sleep = Long.parseLong(commandLine.getOptionValue("sleep", "30000"));
+    long sleep = Long.parseLong(commandLine.getOptionValue("sleep", Long.toString(30000)));
 
     int numServers = Integer.parseInt(commandLine.getOptionValue("numservers", "-1"));
     int numRegions = Integer.parseInt(commandLine.getOptionValue("numregions", "-1"));
