@@ -1,5 +1,4 @@
-/**
- *
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -16,35 +15,28 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.hadoop.hbase.wal;
 
 import java.io.IOException;
-import java.util.Arrays;
 import java.io.InterruptedIOException;
-import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.atomic.AtomicReference;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.hbase.wal.WAL.Reader;
-import org.apache.hadoop.hbase.wal.WALProvider.Writer;
+import org.apache.hadoop.hbase.Abortable;
+import org.apache.hadoop.hbase.client.RegionInfo;
+import org.apache.hadoop.hbase.regionserver.wal.MetricsWAL;
+import org.apache.hadoop.hbase.regionserver.wal.ProtobufLogReader;
 import org.apache.hadoop.hbase.util.CancelableProgressable;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.LeaseNotRecoveredException;
-
-// imports for things that haven't moved from regionserver.wal yet.
-import org.apache.hadoop.hbase.regionserver.wal.MetricsWAL;
-import org.apache.hadoop.hbase.regionserver.wal.ProtobufLogReader;
-import org.apache.hadoop.hbase.regionserver.wal.SequenceFileLogReader;
-import org.apache.hadoop.hbase.regionserver.wal.WALActionsListener;
+import org.apache.hadoop.hbase.wal.WAL.Reader;
+import org.apache.hadoop.hbase.wal.WALProvider.Writer;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Entry point for users of the Write Ahead Log.
@@ -55,30 +47,32 @@ import org.apache.hadoop.hbase.regionserver.wal.WALActionsListener;
  * implementations:
  * <ul>
  *   <li><em>defaultProvider</em> : whatever provider is standard for the hbase version. Currently
- *                                  "filesystem"</li>
+ *                                  "asyncfs"</li>
+ *   <li><em>asyncfs</em> : a provider that will run on top of an implementation of the Hadoop
+ *                             FileSystem interface via an asynchronous client.</li>
  *   <li><em>filesystem</em> : a provider that will run on top of an implementation of the Hadoop
- *                             FileSystem interface, normally HDFS.</li>
+ *                             FileSystem interface via HDFS's synchronous DFSClient.</li>
  *   <li><em>multiwal</em> : a provider that will use multiple "filesystem" wal instances per region
  *                           server.</li>
  * </ul>
  *
- * Alternatively, you may provide a custome implementation of {@link WALProvider} by class name.
+ * Alternatively, you may provide a custom implementation of {@link WALProvider} by class name.
  */
 @InterfaceAudience.Private
 public class WALFactory {
 
-  private static final Log LOG = LogFactory.getLog(WALFactory.class);
+  private static final Logger LOG = LoggerFactory.getLogger(WALFactory.class);
 
   /**
    * Maps between configuration names for providers and implementation classes.
    */
-  static enum Providers {
-    defaultProvider(DefaultWALProvider.class),
-    filesystem(DefaultWALProvider.class),
-    multiwal(RegionGroupingProvider.class);
+  enum Providers {
+    defaultProvider(AsyncFSWALProvider.class),
+    filesystem(FSHLogProvider.class),
+    multiwal(RegionGroupingProvider.class),
+    asyncfs(AsyncFSWALProvider.class);
 
     final Class<? extends WALProvider> clazz;
-
     Providers(Class<? extends WALProvider> clazz) {
       this.clazz = clazz;
     }
@@ -87,20 +81,22 @@ public class WALFactory {
   public static final String WAL_PROVIDER = "hbase.wal.provider";
   static final String DEFAULT_WAL_PROVIDER = Providers.defaultProvider.name();
 
-  static final String META_WAL_PROVIDER = "hbase.wal.meta_provider";
-  static final String DEFAULT_META_WAL_PROVIDER = Providers.defaultProvider.name();
+  public static final String META_WAL_PROVIDER = "hbase.wal.meta_provider";
+
+  public static final String WAL_ENABLED = "hbase.regionserver.hlog.enabled";
 
   final String factoryId;
-  final WALProvider provider;
+  final Abortable abortable;
+  private final WALProvider provider;
   // The meta updates are written to a different wal. If this
   // regionserver holds meta regions, then this ref will be non-null.
   // lazily intialized; most RegionServers don't deal with META
-  final AtomicReference<WALProvider> metaProvider = new AtomicReference<WALProvider>();
+  private final AtomicReference<WALProvider> metaProvider = new AtomicReference<>();
 
   /**
    * Configuration-specified WAL Reader used when a custom reader is requested
    */
-  private final Class<? extends DefaultWALProvider.Reader> logReaderClass;
+  private final Class<? extends AbstractFSWALProvider.Reader> logReaderClass;
 
   /**
    * How long to attempt opening in-recovery wals
@@ -117,32 +113,54 @@ public class WALFactory {
     timeoutMillis = conf.getInt("hbase.hlog.open.timeout", 300000);
     /* TODO Both of these are probably specific to the fs wal provider */
     logReaderClass = conf.getClass("hbase.regionserver.hlog.reader.impl", ProtobufLogReader.class,
-        DefaultWALProvider.Reader.class);
+      AbstractFSWALProvider.Reader.class);
     this.conf = conf;
     // end required early initialization
 
     // this instance can't create wals, just reader/writers.
     provider = null;
     factoryId = SINGLETON_ID;
+    this.abortable = null;
   }
 
-  Class<? extends WALProvider> getProviderClass(String key, String defaultValue) {
+  Providers getDefaultProvider() {
+    return Providers.defaultProvider;
+  }
+
+  public Class<? extends WALProvider> getProviderClass(String key, String defaultValue) {
     try {
-      return Providers.valueOf(conf.get(key, defaultValue)).clazz;
+      Providers provider = Providers.valueOf(conf.get(key, defaultValue));
+
+      // AsyncFSWALProvider is not guaranteed to work on all Hadoop versions, when it's chosen as
+      // the default and we can't use it, we want to fall back to FSHLog which we know works on
+      // all versions.
+      if (provider == getDefaultProvider() && provider.clazz == AsyncFSWALProvider.class
+          && !AsyncFSWALProvider.load()) {
+        // AsyncFSWAL has better performance in most cases, and also uses less resources, we will
+        // try to use it if possible. It deeply hacks into the internal of DFSClient so will be
+        // easily broken when upgrading hadoop.
+        LOG.warn("Failed to load AsyncFSWALProvider, falling back to FSHLogProvider");
+        return FSHLogProvider.class;
+      }
+
+      // N.b. If the user specifically requested AsyncFSWALProvider but their environment doesn't
+      // support using it (e.g. AsyncFSWALProvider.load() == false), we should let this fail and
+      // not fall back to FSHLogProvider.
+      return provider.clazz;
     } catch (IllegalArgumentException exception) {
       // Fall back to them specifying a class name
       // Note that the passed default class shouldn't actually be used, since the above only fails
       // when there is a config value present.
-      return conf.getClass(key, DefaultWALProvider.class, WALProvider.class);
+      return conf.getClass(key, Providers.defaultProvider.clazz, WALProvider.class);
     }
   }
 
-  WALProvider createProvider(Class<? extends WALProvider> clazz,
-      List<WALActionsListener> listeners, String providerId) throws IOException {
+  WALProvider createProvider(Class<? extends WALProvider> clazz, String providerId)
+      throws IOException {
     LOG.info("Instantiating WALProvider of type " + clazz);
     try {
       final WALProvider result = clazz.getDeclaredConstructor().newInstance();
-      result.init(this, conf, listeners, providerId);
+      result.init(this, conf, providerId, this.abortable);
       return result;
     } catch (Exception e) {
       LOG.error("couldn't set up WALProvider, the configured class is " + clazz);
@@ -152,40 +170,43 @@ public class WALFactory {
   }
 
   /**
-   * instantiate a provider from a config property.
-   * requires conf to have already been set (as well as anything the provider might need to read).
+   * instantiate a provider from a config property. requires conf to have already been set (as well
+   * as anything the provider might need to read).
    */
-  WALProvider getProvider(final String key, final String defaultValue,
-      final List<WALActionsListener> listeners, final String providerId) throws IOException {
+  WALProvider getProvider(String key, String defaultValue, String providerId) throws IOException {
     Class<? extends WALProvider> clazz = getProviderClass(key, defaultValue);
-    return createProvider(clazz, listeners, providerId);
+    WALProvider provider = createProvider(clazz, providerId);
+    provider.addWALActionsListener(new MetricsWAL());
+    return provider;
+  }
+
+  public WALFactory(Configuration conf, String factoryId) throws IOException {
+    this(conf, factoryId, null);
   }
 
   /**
    * @param conf must not be null, will keep a reference to read params in later reader/writer
-   *     instances.
-   * @param listeners may be null. will be given to all created wals (and not meta-wals)
-   * @param factoryId a unique identifier for this factory. used i.e. by filesystem implementations
-   *     to make a directory
+   *          instances.
+   * @param abortable the server to abort
    */
-  public WALFactory(final Configuration conf, final List<WALActionsListener> listeners,
-      final String factoryId) throws IOException {
+  public WALFactory(Configuration conf, String factoryId, Abortable abortable) throws IOException {
     // until we've moved reader/writer construction down into providers, this initialization must
     // happen prior to provider initialization, in case they need to instantiate a reader/writer.
     timeoutMillis = conf.getInt("hbase.hlog.open.timeout", 300000);
     /* TODO Both of these are probably specific to the fs wal provider */
     logReaderClass = conf.getClass("hbase.regionserver.hlog.reader.impl", ProtobufLogReader.class,
-        DefaultWALProvider.Reader.class);
+      AbstractFSWALProvider.Reader.class);
     this.conf = conf;
     this.factoryId = factoryId;
+    this.abortable = abortable;
     // end required early initialization
-    if (conf.getBoolean("hbase.regionserver.hlog.enabled", true)) {
-      provider = getProvider(WAL_PROVIDER, DEFAULT_WAL_PROVIDER, listeners, null);
+    if (conf.getBoolean(WAL_ENABLED, true)) {
+      provider = getProvider(WAL_PROVIDER, DEFAULT_WAL_PROVIDER, null);
     } else {
       // special handling of existing configuration behavior.
       LOG.warn("Running with WAL disabled.");
       provider = new DisabledWALProvider();
-      provider.init(this, conf, null, factoryId);
+      provider.init(this, conf, factoryId, null);
     }
   }
 
@@ -227,36 +248,53 @@ public class WALFactory {
     }
   }
 
-  public List<WAL> getWALs() throws IOException {
+  public List<WAL> getWALs() {
     return provider.getWALs();
   }
 
   /**
-   * @param identifier may not be null, contents will not be altered
-   * @param namespace could be null, and will use default namespace if null
+   * Called when we lazily create a hbase:meta WAL OR from ReplicationSourceManager ahead of
+   * creating the first hbase:meta WAL so we can register a listener.
+   * @see #getMetaWALProvider()
    */
-  public WAL getWAL(final byte[] identifier, final byte[] namespace) throws IOException {
-    return provider.getWAL(identifier, namespace);
+  public WALProvider getMetaProvider() throws IOException {
+    for (;;) {
+      WALProvider provider = this.metaProvider.get();
+      if (provider != null) {
+        return provider;
+      }
+      Class<? extends WALProvider> clz = null;
+      if (conf.get(META_WAL_PROVIDER) == null) {
+        try {
+          clz = conf.getClass(WAL_PROVIDER, Providers.defaultProvider.clazz, WALProvider.class);
+        } catch (Throwable t) {
+          // the WAL provider should be an enum. Proceed
+        }
+      }
+      if (clz == null){
+        clz = getProviderClass(META_WAL_PROVIDER, conf.get(WAL_PROVIDER, DEFAULT_WAL_PROVIDER));
+      }
+      provider = createProvider(clz, AbstractFSWALProvider.META_WAL_PROVIDER_ID);
+      if (metaProvider.compareAndSet(null, provider)) {
+        return provider;
+      } else {
+        // someone is ahead of us, close and try again.
+        provider.close();
+      }
+    }
   }
 
   /**
-   * @param identifier may not be null, contents will not be altered
+   * @param region the region which we want to get a WAL for. Could be null.
    */
-  public WAL getMetaWAL(final byte[] identifier) throws IOException {
-    WALProvider metaProvider = this.metaProvider.get();
-    if (null == metaProvider) {
-      final WALProvider temp = getProvider(META_WAL_PROVIDER, DEFAULT_META_WAL_PROVIDER,
-          Collections.<WALActionsListener>singletonList(new MetricsWAL()),
-          DefaultWALProvider.META_WAL_PROVIDER_ID);
-      if (this.metaProvider.compareAndSet(null, temp)) {
-        metaProvider = temp;
-      } else {
-        // reference must now be to a provider created in another thread.
-        temp.close();
-        metaProvider = this.metaProvider.get();
-      }
+  public WAL getWAL(RegionInfo region) throws IOException {
+    // Use different WAL for hbase:meta. Instantiates the meta WALProvider if not already up.
+    if (region != null && region.isMetaRegion() &&
+      region.getReplicaId() == RegionInfo.DEFAULT_REPLICA_ID) {
+      return getMetaProvider().getWAL(region);
+    } else {
+      return provider.getWAL(region);
     }
-    return metaProvider.getWAL(identifier, null);
   }
 
   public Reader createReader(final FileSystem fs, final Path path) throws IOException {
@@ -268,19 +306,16 @@ public class WALFactory {
    * to reopen it multiple times, use {@link WAL.Reader#reset()} instead of this method
    * then just seek back to the last known good position.
    * @return A WAL reader.  Close when done with it.
-   * @throws IOException
    */
   public Reader createReader(final FileSystem fs, final Path path,
       CancelableProgressable reporter) throws IOException {
     return createReader(fs, path, reporter, true);
   }
 
-  public Reader createReader(final FileSystem fs, final Path path,
-      CancelableProgressable reporter, boolean allowCustom)
-      throws IOException {
-    Class<? extends DefaultWALProvider.Reader> lrClass =
+  public Reader createReader(final FileSystem fs, final Path path, CancelableProgressable reporter,
+      boolean allowCustom) throws IOException {
+    Class<? extends AbstractFSWALProvider.Reader> lrClass =
         allowCustom ? logReaderClass : ProtobufLogReader.class;
-
     try {
       // A wal file could be under recovery, so it may take several
       // tries to get it open. Instead of claiming it is corrupted, retry
@@ -288,38 +323,15 @@ public class WALFactory {
       long startWaiting = EnvironmentEdgeManager.currentTime();
       long openTimeout = timeoutMillis + startWaiting;
       int nbAttempt = 0;
-      FSDataInputStream stream = null;
-      DefaultWALProvider.Reader reader = null;
+      AbstractFSWALProvider.Reader reader = null;
       while (true) {
         try {
-          if (lrClass != ProtobufLogReader.class) {
-            // User is overriding the WAL reader, let them.
-            reader = lrClass.getDeclaredConstructor().newInstance();
-            reader.init(fs, path, conf, null);
-            return reader;
-          } else {
-            stream = fs.open(path);
-            // Note that zero-length file will fail to read PB magic, and attempt to create
-            // a non-PB reader and fail the same way existing code expects it to. If we get
-            // rid of the old reader entirely, we need to handle 0-size files differently from
-            // merely non-PB files.
-            byte[] magic = new byte[ProtobufLogReader.PB_WAL_MAGIC.length];
-            boolean isPbWal = (stream.read(magic) == magic.length)
-                && Arrays.equals(magic, ProtobufLogReader.PB_WAL_MAGIC);
-            reader =
-                isPbWal ? new ProtobufLogReader() : new SequenceFileLogReader();
-            reader.init(fs, path, conf, stream);
-            return reader;
-          }
+          reader = lrClass.getDeclaredConstructor().newInstance();
+          reader.init(fs, path, conf, null);
+          return reader;
         } catch (Exception e) {
-          if (stream != null) {
-            try {
-              stream.close();
-            } catch (IOException exception) {
-              LOG.warn("Could not close DefaultWALProvider.Reader" + exception.getMessage());
-              LOG.debug("exception details", exception);
-            }
-          }
+          // catch Exception so that we close reader for all exceptions. If we don't
+          // close the reader, we leak a socket.
           if (reader != null) {
             try {
               reader.close();
@@ -328,11 +340,14 @@ public class WALFactory {
               LOG.debug("exception details", exception);
             }
           }
+
+          // Only inspect the Exception to consider retry when it's an IOException
           if (e instanceof IOException) {
             String msg = e.getMessage();
-            if (msg != null && (msg.contains("Cannot obtain block length")
-                || msg.contains("Could not obtain the last block")
-                || msg.matches("Blocklist for [^ ]* has changed.*"))) {
+            if (msg != null
+                && (msg.contains("Cannot obtain block length")
+                    || msg.contains("Could not obtain the last block") || msg
+                      .matches("Blocklist for [^ ]* has changed.*"))) {
               if (++nbAttempt == 1) {
                 LOG.warn("Lease should have recovered. This is not expected. Will retry", e);
               }
@@ -341,8 +356,7 @@ public class WALFactory {
               }
               if (nbAttempt > 2 && openTimeout < EnvironmentEdgeManager.currentTime()) {
                 LOG.error("Can't open after " + nbAttempt + " attempts and "
-                    + (EnvironmentEdgeManager.currentTime() - startWaiting)
-                    + "ms " + " for " + path);
+                    + (EnvironmentEdgeManager.currentTime() - startWaiting) + "ms " + " for " + path);
               } else {
                 try {
                   Thread.sleep(nbAttempt < 3 ? 500 : 1000);
@@ -354,8 +368,6 @@ public class WALFactory {
                 }
               }
               throw new LeaseNotRecoveredException(e);
-            } else {
-              throw e;
             }
           }
 
@@ -372,32 +384,34 @@ public class WALFactory {
 
   /**
    * Create a writer for the WAL.
-   * should be package-private. public only for tests and
+   * Uses defaults.
+   * <p>
+   * Should be package-private. public only for tests and
    * {@link org.apache.hadoop.hbase.regionserver.wal.Compressor}
-   * @return A WAL writer.  Close when done with it.
-   * @throws IOException
+   * @return A WAL writer. Close when done with it.
    */
   public Writer createWALWriter(final FileSystem fs, final Path path) throws IOException {
-    return DefaultWALProvider.createWriter(conf, fs, path, false);
+    return FSHLogProvider.createWriter(conf, fs, path, false);
   }
 
   /**
-   * should be package-private, visible for recovery testing.
+   * Should be package-private, visible for recovery testing.
+   * Uses defaults.
    * @return an overwritable writer for recovered edits. caller should close.
    */
   public Writer createRecoveredEditsWriter(final FileSystem fs, final Path path)
       throws IOException {
-    return DefaultWALProvider.createWriter(conf, fs, path, true);
+    return FSHLogProvider.createWriter(conf, fs, path, true);
   }
 
   // These static methods are currently used where it's impractical to
   // untangle the reliance on state in the filesystem. They rely on singleton
   // WALFactory that just provides Reader / Writers.
   // For now, first Configuration object wins. Practically this just impacts the reader/writer class
-  private static final AtomicReference<WALFactory> singleton = new AtomicReference<WALFactory>();
+  private static final AtomicReference<WALFactory> singleton = new AtomicReference<>();
   private static final String SINGLETON_ID = WALFactory.class.getName();
   
-  // public only for FSHLog and UpgradeTo96
+  // Public only for FSHLog
   public static WALFactory getInstance(Configuration configuration) {
     WALFactory factory = singleton.get();
     if (null == factory) {
@@ -450,28 +464,38 @@ public class WALFactory {
 
   /**
    * If you already have a WALFactory, you should favor the instance method.
+   * Uses defaults.
    * @return a Writer that will overwrite files. Caller must close.
    */
   static Writer createRecoveredEditsWriter(final FileSystem fs, final Path path,
       final Configuration configuration)
       throws IOException {
-    return DefaultWALProvider.createWriter(configuration, fs, path, true);
+    return FSHLogProvider.createWriter(configuration, fs, path, true);
   }
 
   /**
    * If you already have a WALFactory, you should favor the instance method.
+   * Uses defaults.
    * @return a writer that won't overwrite files. Caller must close.
    */
   public static Writer createWALWriter(final FileSystem fs, final Path path,
       final Configuration configuration)
       throws IOException {
-    return DefaultWALProvider.createWriter(configuration, fs, path, false);
+    return FSHLogProvider.createWriter(configuration, fs, path, false);
+  }
+
+  public String getFactoryId() {
+    return factoryId;
   }
 
   public final WALProvider getWALProvider() {
     return this.provider;
   }
 
+  /**
+   * @return Current metaProvider... may be null if not yet initialized.
+   * @see #getMetaProvider()
+   */
   public final WALProvider getMetaWALProvider() {
     return this.metaProvider.get();
   }
