@@ -21,29 +21,31 @@ package org.apache.hadoop.hbase;
 import java.io.IOException;
 import java.security.PrivilegedAction;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
-import org.apache.hadoop.hbase.classification.InterfaceStability;
+import java.util.Set;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.hbase.client.RegionReplicaUtil;
 import org.apache.hadoop.hbase.master.HMaster;
-import org.apache.hadoop.hbase.protobuf.generated.AdminProtos.AdminService;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ClientService;
-import org.apache.hadoop.hbase.protobuf.generated.MasterProtos.MasterService;
-import org.apache.hadoop.hbase.protobuf.generated.RegionServerStatusProtos.RegionServerStartupResponse;
 import org.apache.hadoop.hbase.regionserver.HRegion;
+import org.apache.hadoop.hbase.regionserver.HRegion.FlushResult;
 import org.apache.hadoop.hbase.regionserver.HRegionServer;
 import org.apache.hadoop.hbase.regionserver.Region;
 import org.apache.hadoop.hbase.security.User;
 import org.apache.hadoop.hbase.test.MetricsAssertHelper;
-import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.JVMClusterUtil;
 import org.apache.hadoop.hbase.util.JVMClusterUtil.MasterThread;
 import org.apache.hadoop.hbase.util.JVMClusterUtil.RegionServerThread;
 import org.apache.hadoop.hbase.util.Threads;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import org.apache.hadoop.hbase.shaded.protobuf.generated.AdminProtos.AdminService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ClientService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.MasterProtos.MasterService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.RegionServerStatusProtos.RegionServerStartupResponse;
 
 /**
  * This class creates a single process HBase cluster.
@@ -52,9 +54,8 @@ import org.apache.hadoop.hbase.util.Threads;
  * each and will close down their instance on the way out.
  */
 @InterfaceAudience.Public
-@InterfaceStability.Evolving
 public class MiniHBaseCluster extends HBaseCluster {
-  private static final Log LOG = LogFactory.getLog(MiniHBaseCluster.class.getName());
+  private static final Logger LOG = LoggerFactory.getLogger(MiniHBaseCluster.class.getName());
   public LocalHBaseCluster hbaseCluster;
   private static int index;
 
@@ -81,23 +82,39 @@ public class MiniHBaseCluster extends HBaseCluster {
     this(conf, numMasters, numRegionServers, null, null);
   }
 
+  /**
+   * Start a MiniHBaseCluster.
+   * @param conf Configuration to be used for cluster
+   * @param numMasters initial number of masters to start.
+   * @param numRegionServers initial number of region servers to start.
+   */
   public MiniHBaseCluster(Configuration conf, int numMasters, int numRegionServers,
          Class<? extends HMaster> masterClass,
          Class<? extends MiniHBaseCluster.MiniHBaseClusterRegionServer> regionserverClass)
       throws IOException, InterruptedException {
+    this(conf, numMasters, 0, numRegionServers, null, masterClass, regionserverClass);
+  }
+
+  /**
+   * @param rsPorts Ports that RegionServer should use; pass ports if you want to test cluster
+   *   restart where for sure the regionservers come up on same address+port (but
+   *   just with different startcode); by default mini hbase clusters choose new
+   *   arbitrary ports on each cluster start.
+   * @throws IOException
+   * @throws InterruptedException
+   */
+  public MiniHBaseCluster(Configuration conf, int numMasters, int numAlwaysStandByMasters,
+         int numRegionServers, List<Integer> rsPorts, Class<? extends HMaster> masterClass,
+         Class<? extends MiniHBaseCluster.MiniHBaseClusterRegionServer> regionserverClass)
+      throws IOException, InterruptedException {
     super(conf);
-    if (conf.getBoolean(LocalHBaseCluster.ASSIGN_RANDOM_PORTS, true)) {
-      conf.set(HConstants.MASTER_PORT, "0");
-      if (conf.getInt(HConstants.MASTER_INFO_PORT, 0) != -1) {
-        conf.set(HConstants.MASTER_INFO_PORT, "0");
-      }
-    }
 
     // Hadoop 2
     CompatibilityFactory.getInstance(MetricsAssertHelper.class).init();
 
-    init(numMasters, numRegionServers, masterClass, regionserverClass);
-    this.initialClusterStatus = getClusterStatus();
+    init(numMasters, numAlwaysStandByMasters, numRegionServers, rsPorts, masterClass,
+        regionserverClass);
+    this.initialClusterStatus = getClusterMetrics();
   }
 
   public Configuration getConfiguration() {
@@ -113,10 +130,16 @@ public class MiniHBaseCluster extends HBaseCluster {
   public static class MiniHBaseClusterRegionServer extends HRegionServer {
     private Thread shutdownThread = null;
     private User user = null;
+    /**
+     * List of RegionServers killed so far. ServerName also comprises startCode of a server,
+     * so any restarted instances of the same server will have different ServerName and will not
+     * coincide with past dead ones. So there's no need to cleanup this list.
+     */
+    static Set<ServerName> killedServers = new HashSet<>();
 
-    public MiniHBaseClusterRegionServer(Configuration conf, CoordinatedStateManager cp)
+    public MiniHBaseClusterRegionServer(Configuration conf)
         throws IOException, InterruptedException {
-      super(conf, cp);
+      super(conf);
       this.user = User.getCurrent();
     }
 
@@ -139,7 +162,8 @@ public class MiniHBaseCluster extends HBaseCluster {
     @Override
     public void run() {
       try {
-        this.user.runAs(new PrivilegedAction<Object>(){
+        this.user.runAs(new PrivilegedAction<Object>() {
+          @Override
           public Object run() {
             runRegionServer();
             return null;
@@ -161,13 +185,15 @@ public class MiniHBaseCluster extends HBaseCluster {
     }
 
     @Override
-    public void kill() {
+    protected void kill() {
+      killedServers.add(getServerName());
       super.kill();
     }
 
     @Override
     public void abort(final String reason, final Throwable cause) {
       this.user.runAs(new PrivilegedAction<Object>() {
+        @Override
         public Object run() {
           abortRegionServer(reason, cause);
           return null;
@@ -203,9 +229,9 @@ public class MiniHBaseCluster extends HBaseCluster {
     }
   }
 
-  private void init(final int nMasterNodes, final int nRegionNodes,
-                 Class<? extends HMaster> masterClass,
-                 Class<? extends MiniHBaseCluster.MiniHBaseClusterRegionServer> regionserverClass)
+  private void init(final int nMasterNodes, final int numAlwaysStandByMasters,
+      final int nRegionNodes, List<Integer> rsPorts, Class<? extends HMaster> masterClass,
+      Class<? extends MiniHBaseCluster.MiniHBaseClusterRegionServer> regionserverClass)
   throws IOException, InterruptedException {
     try {
       if (masterClass == null){
@@ -216,12 +242,15 @@ public class MiniHBaseCluster extends HBaseCluster {
       }
 
       // start up a LocalHBaseCluster
-      hbaseCluster = new LocalHBaseCluster(conf, nMasterNodes, 0,
+      hbaseCluster = new LocalHBaseCluster(conf, nMasterNodes, numAlwaysStandByMasters, 0,
           masterClass, regionserverClass);
 
       // manually add the regionservers as other users
-      for (int i=0; i<nRegionNodes; i++) {
+      for (int i = 0; i < nRegionNodes; i++) {
         Configuration rsConf = HBaseConfiguration.create(conf);
+        if (rsPorts != null) {
+          rsConf.setInt(HConstants.REGIONSERVER_PORT, rsPorts.get(i));
+        }
         User user = HBaseTestingUtility.getDifferentUser(rsConf,
             ".hfs."+index++);
         hbaseCluster.addRegionServer(rsConf, i, user);
@@ -240,7 +269,9 @@ public class MiniHBaseCluster extends HBaseCluster {
 
   @Override
   public void startRegionServer(String hostname, int port) throws IOException {
-    this.startRegionServer();
+    final Configuration newConf = HBaseConfiguration.create(conf);
+    newConf.setInt(HConstants.REGIONSERVER_PORT, port);
+    startRegionServer(newConf);
   }
 
   @Override
@@ -254,6 +285,11 @@ public class MiniHBaseCluster extends HBaseCluster {
     }
   }
 
+  @Override
+  public boolean isKilledRS(ServerName serverName) {
+    return MiniHBaseClusterRegionServer.killedServers.contains(serverName);
+  }
+
   @Override
   public void stopRegionServer(ServerName serverName) throws IOException {
     stopRegionServer(getRegionServerIndex(serverName));
@@ -380,12 +416,17 @@ public class MiniHBaseCluster extends HBaseCluster {
   public JVMClusterUtil.RegionServerThread startRegionServer()
       throws IOException {
     final Configuration newConf = HBaseConfiguration.create(conf);
+    return startRegionServer(newConf);
+  }
+
+  private JVMClusterUtil.RegionServerThread startRegionServer(Configuration configuration)
+      throws IOException {
     User rsUser =
-        HBaseTestingUtility.getDifferentUser(newConf, ".hfs."+index++);
+        HBaseTestingUtility.getDifferentUser(configuration, ".hfs."+index++);
     JVMClusterUtil.RegionServerThread t =  null;
     try {
       t = hbaseCluster.addRegionServer(
-          newConf, hbaseCluster.getRegionServers().size(), rsUser);
+          configuration, hbaseCluster.getRegionServers().size(), rsUser);
       t.start();
       t.waitForServerOnline();
     } catch (InterruptedException ie) {
@@ -402,15 +443,15 @@ public class MiniHBaseCluster extends HBaseCluster {
    * @return New RegionServerThread
    */
   public JVMClusterUtil.RegionServerThread startRegionServerAndWait(long timeout)
-    throws IOException {
+      throws IOException {
 
     JVMClusterUtil.RegionServerThread t =  startRegionServer();
     ServerName rsServerName = t.getRegionServer().getServerName();
 
-    long start = EnvironmentEdgeManager.currentTime();
-    while ((EnvironmentEdgeManager.currentTime() - start) < timeout) {
-      ClusterStatus clusterStatus = getMaster().getClusterStatus();
-      if (clusterStatus != null && clusterStatus.getLiveServersLoad().containsKey(rsServerName)) {
+    long start = System.currentTimeMillis();
+    ClusterStatus clusterStatus = getClusterStatus();
+    while ((System.currentTimeMillis() - start) < timeout) {
+      if (clusterStatus != null && clusterStatus.getServers().contains(rsServerName)) {
         return t;
       }
       Threads.sleep(100);
@@ -465,12 +506,12 @@ public class MiniHBaseCluster extends HBaseCluster {
   /**
    * Suspend the specified region server
    * @param serverNumber Used as index into a list.
-   * @return server instance that was suspended.
+   * @return
    */
   public JVMClusterUtil.RegionServerThread suspendRegionServer(int serverNumber) {
     JVMClusterUtil.RegionServerThread server =
         hbaseCluster.getRegionServers().get(serverNumber);
-    LOG.info("Suspending " + server.toString());
+    LOG.info("Suspending {}", server.toString());
     server.suspend();
     return server;
   }
@@ -478,12 +519,12 @@ public class MiniHBaseCluster extends HBaseCluster {
   /**
    * Resume the specified region server
    * @param serverNumber Used as index into a list.
-   * @return server instance that was resumed.
+   * @return
    */
   public JVMClusterUtil.RegionServerThread resumeRegionServer(int serverNumber) {
     JVMClusterUtil.RegionServerThread server =
         hbaseCluster.getRegionServers().get(serverNumber);
-    LOG.info("Resuming " + server.toString());
+    LOG.info("Resuming {}", server.toString());
     server.resume();
     return server;
   }
@@ -502,7 +543,6 @@ public class MiniHBaseCluster extends HBaseCluster {
   /**
    * Starts a master thread running
    *
-   * @throws IOException
    * @return New RegionServerThread
    */
   public JVMClusterUtil.MasterThread startMaster() throws IOException {
@@ -517,6 +557,8 @@ public class MiniHBaseCluster extends HBaseCluster {
     } catch (InterruptedException ie) {
       throw new IOException("Interrupted adding master to cluster", ie);
     }
+    conf.set(HConstants.MASTER_ADDRS_KEY,
+        hbaseCluster.getConfiguration().get(HConstants.MASTER_ADDRS_KEY));
     return t;
   }
 
@@ -524,6 +566,7 @@ public class MiniHBaseCluster extends HBaseCluster {
    * Returns the current active master, if available.
    * @return the active HMaster, null if none is active.
    */
+  @Override
   public MasterService.BlockingInterface getMasterAdminService() {
     return this.hbaseCluster.getActiveMaster().getMasterRpcServices();
   }
@@ -615,6 +658,7 @@ public class MiniHBaseCluster extends HBaseCluster {
    *         masters left.
    * @throws InterruptedException
    */
+  @Override
   public boolean waitForActiveAndReadyMaster(long timeout) throws IOException {
     List<JVMClusterUtil.MasterThread> mts;
     long start = System.currentTimeMillis();
@@ -654,9 +698,8 @@ public class MiniHBaseCluster extends HBaseCluster {
 
   /**
    * Shut down the mini HBase cluster
-   * @throws IOException
    */
-  @SuppressWarnings("deprecation")
+  @Override
   public void shutdown() throws IOException {
     if (this.hbaseCluster != null) {
       this.hbaseCluster.shutdown();
@@ -667,35 +710,55 @@ public class MiniHBaseCluster extends HBaseCluster {
   public void close() throws IOException {
   }
 
-  @Override
+  /**
+   * @deprecated As of release 2.0.0, this will be removed in HBase 3.0.0
+   *             Use {@link #getClusterMetrics()} instead.
+   */
+  @Deprecated
   public ClusterStatus getClusterStatus() throws IOException {
     HMaster master = getMaster();
-    return master == null ? null : master.getClusterStatus();
+    return master == null ? null : new ClusterStatus(master.getClusterMetrics());
+  }
+
+  @Override
+  public ClusterMetrics getClusterMetrics() throws IOException {
+    HMaster master = getMaster();
+    return master == null ? null : master.getClusterMetrics();
+  }
+
+  private void executeFlush(HRegion region) throws IOException {
+    if (!RegionReplicaUtil.isDefaultReplica(region.getRegionInfo())) {
+      return;
+    }
+    // retry 5 times if we can not flush
+    for (int i = 0; i < 5; i++) {
+      FlushResult result = region.flush(true);
+      if (result.getResult() != FlushResult.Result.CANNOT_FLUSH) {
+        return;
+      }
+      Threads.sleep(1000);
+    }
   }
 
   /**
    * Call flushCache on all regions on all participating regionservers.
-   * @throws IOException
    */
   public void flushcache() throws IOException {
-    for (JVMClusterUtil.RegionServerThread t:
-        this.hbaseCluster.getRegionServers()) {
-      for(Region r: t.getRegionServer().getOnlineRegionsLocalContext()) {
-        r.flush(true);
+    for (JVMClusterUtil.RegionServerThread t : this.hbaseCluster.getRegionServers()) {
+      for (HRegion r : t.getRegionServer().getOnlineRegionsLocalContext()) {
+        executeFlush(r);
       }
     }
   }
 
   /**
    * Call flushCache on all regions of the specified table.
-   * @throws IOException
    */
   public void flushcache(TableName tableName) throws IOException {
-    for (JVMClusterUtil.RegionServerThread t:
-        this.hbaseCluster.getRegionServers()) {
-      for(Region r: t.getRegionServer().getOnlineRegionsLocalContext()) {
-        if(r.getTableDesc().getTableName().equals(tableName)) {
-          r.flush(true);
+    for (JVMClusterUtil.RegionServerThread t : this.hbaseCluster.getRegionServers()) {
+      for (HRegion r : t.getRegionServer().getOnlineRegionsLocalContext()) {
+        if (r.getTableDescriptor().getTableName().equals(tableName)) {
+          executeFlush(r);
         }
       }
     }
@@ -706,10 +769,11 @@ public class MiniHBaseCluster extends HBaseCluster {
    * @throws IOException
    */
   public void compact(boolean major) throws IOException {
-    for (JVMClusterUtil.RegionServerThread t:
-        this.hbaseCluster.getRegionServers()) {
-      for(Region r: t.getRegionServer().getOnlineRegionsLocalContext()) {
-        r.compact(major);
+    for (JVMClusterUtil.RegionServerThread t : this.hbaseCluster.getRegionServers()) {
+      for (HRegion r : t.getRegionServer().getOnlineRegionsLocalContext()) {
+        if (RegionReplicaUtil.isDefaultReplica(r.getRegionInfo())) {
+          r.compact(major);
+        }
       }
     }
   }
@@ -719,18 +783,27 @@ public class MiniHBaseCluster extends HBaseCluster {
    * @throws IOException
    */
   public void compact(TableName tableName, boolean major) throws IOException {
-    for (JVMClusterUtil.RegionServerThread t:
-        this.hbaseCluster.getRegionServers()) {
-      for(Region r: t.getRegionServer().getOnlineRegionsLocalContext()) {
-        if(r.getTableDesc().getTableName().equals(tableName)) {
-          r.compact(major);
+    for (JVMClusterUtil.RegionServerThread t : this.hbaseCluster.getRegionServers()) {
+      for (HRegion r : t.getRegionServer().getOnlineRegionsLocalContext()) {
+        if (r.getTableDescriptor().getTableName().equals(tableName)) {
+          if (RegionReplicaUtil.isDefaultReplica(r.getRegionInfo())) {
+            r.compact(major);
+          }
         }
       }
     }
   }
 
   /**
-   * @return List of region server threads.
+   * @return Number of live region servers in the cluster currently.
+   */
+  public int getNumLiveRegionServers() {
+    return this.hbaseCluster.getLiveRegionServers().size();
+  }
+
+  /**
+   * @return List of region server threads. Does not return the master even though it is also
+   * a region server.
    */
   public List<JVMClusterUtil.RegionServerThread> getRegionServerThreads() {
     return this.hbaseCluster.getRegionServers();
@@ -753,13 +826,10 @@ public class MiniHBaseCluster extends HBaseCluster {
   }
 
   public HRegionServer getRegionServer(ServerName serverName) {
-    for (RegionServerThread t : hbaseCluster.getRegionServers()) {
-      HRegionServer r = t.getRegionServer();
-      if (r.getServerName().equals(serverName)) {
-        return r;
-      }
-    }
-    return null;
+    return hbaseCluster.getRegionServers().stream()
+        .map(t -> t.getRegionServer())
+        .filter(r -> r.getServerName().equals(serverName))
+        .findFirst().orElse(null);
   }
 
   public List<HRegion> getRegions(byte[] tableName) {
@@ -767,11 +837,11 @@ public class MiniHBaseCluster extends HBaseCluster {
   }
 
   public List<HRegion> getRegions(TableName tableName) {
-    List<HRegion> ret = new ArrayList<HRegion>();
+    List<HRegion> ret = new ArrayList<>();
     for (JVMClusterUtil.RegionServerThread rst : getRegionServerThreads()) {
       HRegionServer hrs = rst.getRegionServer();
       for (Region region : hrs.getOnlineRegionsLocalContext()) {
-        if (region.getTableDesc().getTableName().equals(tableName)) {
+        if (region.getTableDescriptor().getTableName().equals(tableName)) {
           ret.add((HRegion)region);
         }
       }
@@ -851,12 +921,22 @@ public class MiniHBaseCluster extends HBaseCluster {
    * impossible to bring the mini-cluster back for clean shutdown.
    */
   public void killAll() {
+    // Do backups first.
+    MasterThread activeMaster = null;
+    for (MasterThread masterThread : getMasterThreads()) {
+      if (!masterThread.getMaster().isActiveMaster()) {
+        masterThread.getMaster().abort("killAll");
+      } else {
+        activeMaster = masterThread;
+      }
+    }
+    // Do active after.
+    if (activeMaster != null) {
+      activeMaster.getMaster().abort("killAll");
+    }
     for (RegionServerThread rst : getRegionServerThreads()) {
       rst.getRegionServer().abort("killAll");
     }
-    for (MasterThread masterThread : getMasterThreads()) {
-      masterThread.getMaster().abort("killAll", new Throwable());
-    }
   }
 
   @Override
@@ -865,11 +945,11 @@ public class MiniHBaseCluster extends HBaseCluster {
   }
 
   public List<HRegion> findRegionsForTable(TableName tableName) {
-    ArrayList<HRegion> ret = new ArrayList<HRegion>();
+    ArrayList<HRegion> ret = new ArrayList<>();
     for (JVMClusterUtil.RegionServerThread rst : getRegionServerThreads()) {
       HRegionServer hrs = rst.getRegionServer();
-      for (Region region : hrs.getOnlineRegions(tableName)) {
-        if (region.getTableDesc().getTableName().equals(tableName)) {
+      for (Region region : hrs.getRegions(tableName)) {
+        if (region.getTableDescriptor().getTableName().equals(tableName)) {
           ret.add((HRegion)region);
         }
       }
