@@ -865,6 +865,14 @@ public class TestHFile  {
     testReaderCombinedCache("TinyLfu");
   }
 
+  /**
+   * Test case for CombinedBlockCache with AdaptiveLRU as L1 cache
+   */
+  @Test
+  public void testReaderWithAdaptiveLruCombinedBlockCache() throws Exception {
+    testReaderCombinedCache("AdaptiveLRU");
+  }
+
   /**
    * Test case for CombinedBlockCache with AdaptiveLRU as L1 cache
    */
