@@ -1,5 +1,4 @@
 /**
- *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,24 +18,26 @@
 package org.apache.hadoop.hbase.protobuf;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-
+import static org.junit.Assert.assertTrue;
 import com.google.protobuf.ByteString;
 import java.io.IOException;
-import java.util.Collections;
-import java.util.List;
-
+import java.nio.ByteBuffer;
+import org.apache.hadoop.hbase.ByteBufferKeyValue;
 import org.apache.hadoop.hbase.Cell;
+import org.apache.hadoop.hbase.CellBuilderType;
+import org.apache.hadoop.hbase.CellComparatorImpl;
+import org.apache.hadoop.hbase.ExtendedCellBuilderFactory;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.HConstants;
-import org.apache.hadoop.hbase.HRegionInfo;
 import org.apache.hadoop.hbase.KeyValue;
 import org.apache.hadoop.hbase.ServerName;
-import org.apache.hadoop.hbase.Tag;
 import org.apache.hadoop.hbase.client.Append;
 import org.apache.hadoop.hbase.client.Delete;
 import org.apache.hadoop.hbase.client.Get;
 import org.apache.hadoop.hbase.client.Increment;
 import org.apache.hadoop.hbase.client.Put;
+import org.apache.hadoop.hbase.client.RegionInfoBuilder;
+import org.apache.hadoop.hbase.io.TimeRange;
 import org.apache.hadoop.hbase.master.RegionState;
 import org.apache.hadoop.hbase.protobuf.generated.CellProtos;
 import org.apache.hadoop.hbase.protobuf.generated.ClientProtos;
@@ -47,32 +48,35 @@ import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutationProto.Col
 import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutationProto.DeleteType;
 import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutationProto.MutationType;
 import org.apache.hadoop.hbase.protobuf.generated.HBaseProtos.NameBytesPair;
-import org.apache.hadoop.hbase.protobuf.generated.ZooKeeperProtos.MetaRegionServer;
+import org.apache.hadoop.hbase.testclassification.MiscTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
-import org.apache.hadoop.hbase.util.ByteStringer;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ZooKeeperProtos.MetaRegionServer;
 
 /**
  * Class to test ProtobufUtil.
  */
-@Category(SmallTests.class)
+@Category({ MiscTests.class, SmallTests.class})
 public class TestProtobufUtil {
-  private static final String TAG_STR = "tag-1";
-  private static final byte TAG_TYPE = (byte)10;
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestProtobufUtil.class);
 
   @Test
   public void testException() throws IOException {
     NameBytesPair.Builder builder = NameBytesPair.newBuilder();
     final String omg = "OMG!!!";
     builder.setName("java.io.IOException");
-    builder.setValue(ByteStringer.wrap(Bytes.toBytes(omg)));
+    builder.setValue(ByteString.copyFrom(Bytes.toBytes(omg)));
     Throwable t = ProtobufUtil.toException(builder.build());
     assertEquals(omg, t.getMessage());
     builder.clear();
     builder.setName("org.apache.hadoop.ipc.RemoteException");
-    builder.setValue(ByteStringer.wrap(Bytes.toBytes(omg)));
+    builder.setValue(ByteString.copyFrom(Bytes.toBytes(omg)));
     t = ProtobufUtil.toException(builder.build());
     assertEquals(omg, t.getMessage());
   }
@@ -96,7 +100,6 @@ public class TestProtobufUtil {
     columnBuilder.setFamily(ByteString.copyFromUtf8("f2"));
     getBuilder.addColumn(columnBuilder.build());
     getBuilder.setLoadColumnFamiliesOnDemand(true);
-
     ClientProtos.Get proto = getBuilder.build();
     // default fields
     assertEquals(1, proto.getMaxVersions());
@@ -106,6 +109,7 @@ public class TestProtobufUtil {
     getBuilder = ClientProtos.Get.newBuilder(proto);
     getBuilder.setMaxVersions(1);
     getBuilder.setCacheBlocks(true);
+    getBuilder.setTimeRange(ProtobufUtil.toTimeRange(TimeRange.allTime()));
 
     Get get = ProtobufUtil.toGet(proto);
     assertEquals(getBuilder.build(), ProtobufUtil.toGet(get));
@@ -118,40 +122,59 @@ public class TestProtobufUtil {
    */
   @Test
   public void testAppend() throws IOException {
-    long timeStamp = 111111;
-    MutationProto.Builder mutateBuilder = MutationProto.newBuilder();
-    mutateBuilder.setRow(ByteString.copyFromUtf8("row"));
-    mutateBuilder.setMutateType(MutationType.APPEND);
-    mutateBuilder.setTimestamp(timeStamp);
-    ColumnValue.Builder valueBuilder = ColumnValue.newBuilder();
-    valueBuilder.setFamily(ByteString.copyFromUtf8("f1"));
-    QualifierValue.Builder qualifierBuilder = QualifierValue.newBuilder();
-    qualifierBuilder.setQualifier(ByteString.copyFromUtf8("c1"));
-    qualifierBuilder.setValue(ByteString.copyFromUtf8("v1"));
-    qualifierBuilder.setTimestamp(timeStamp);
-    valueBuilder.addQualifierValue(qualifierBuilder.build());
-    qualifierBuilder.setQualifier(ByteString.copyFromUtf8("c2"));
-    qualifierBuilder.setValue(ByteString.copyFromUtf8("v2"));
-    valueBuilder.addQualifierValue(qualifierBuilder.build());
-    qualifierBuilder.setTimestamp(timeStamp);
-    mutateBuilder.addColumnValue(valueBuilder.build());
-
-    MutationProto proto = mutateBuilder.build();
+    MutationProto proto = getAppendMutation(111111L);
     // default fields
     assertEquals(MutationProto.Durability.USE_DEFAULT, proto.getDurability());
 
     // set the default value for equal comparison
-    mutateBuilder = MutationProto.newBuilder(proto);
+    MutationProto.Builder mutateBuilder = MutationProto.newBuilder(proto);
     mutateBuilder.setDurability(MutationProto.Durability.USE_DEFAULT);
 
     Append append = ProtobufUtil.toAppend(proto, null);
 
     // append always use the latest timestamp,
     // reset the timestamp to the original mutate
-    mutateBuilder.setTimestamp(append.getTimeStamp());
+    mutateBuilder.setTimestamp(append.getTimestamp());
+    mutateBuilder.setTimeRange(ProtobufUtil.toTimeRange(append.getTimeRange()));
     assertEquals(mutateBuilder.build(), ProtobufUtil.toMutation(MutationType.APPEND, append));
   }
 
+  /**
+   * Older clients may not send along a timestamp in the MutationProto. Check that we
+   * default correctly.
+   */
+  @Test
+  public void testAppendNoTimestamp() throws IOException {
+    MutationProto mutation = getAppendMutation(null);
+    Append append = ProtobufUtil.toAppend(mutation, null);
+    assertEquals(HConstants.LATEST_TIMESTAMP, append.getTimestamp());
+    append.getFamilyCellMap().values().forEach(cells -> cells.forEach(cell -> assertEquals(HConstants.LATEST_TIMESTAMP, cell.getTimestamp())));
+  }
+
+  private MutationProto getAppendMutation(Long timestamp) {
+    MutationProto.Builder mutateBuilder = MutationProto.newBuilder();
+    mutateBuilder.setRow(ByteString.copyFromUtf8("row"));
+    mutateBuilder.setMutateType(MutationType.APPEND);
+    if (timestamp != null) {
+      mutateBuilder.setTimestamp(timestamp);
+    }
+    ColumnValue.Builder valueBuilder = ColumnValue.newBuilder();
+    valueBuilder.setFamily(ByteString.copyFromUtf8("f1"));
+    QualifierValue.Builder qualifierBuilder = QualifierValue.newBuilder();
+    qualifierBuilder.setQualifier(ByteString.copyFromUtf8("c1"));
+    qualifierBuilder.setValue(ByteString.copyFromUtf8("v1"));
+    if (timestamp != null) {
+      qualifierBuilder.setTimestamp(timestamp);
+    }
+    valueBuilder.addQualifierValue(qualifierBuilder.build());
+    qualifierBuilder.setQualifier(ByteString.copyFromUtf8("c2"));
+    qualifierBuilder.setValue(ByteString.copyFromUtf8("v2"));
+    valueBuilder.addQualifierValue(qualifierBuilder.build());
+    mutateBuilder.addColumnValue(valueBuilder.build());
+
+    return mutateBuilder.build();
+  }
+
   /**
    * Test Delete Mutate conversions.
    *
@@ -206,6 +229,36 @@ public class TestProtobufUtil {
    */
   @Test
   public void testIncrement() throws IOException {
+    MutationProto proto = getIncrementMutation(111111L);
+    // default fields
+    assertEquals(MutationProto.Durability.USE_DEFAULT, proto.getDurability());
+
+    // set the default value for equal comparison
+    MutationProto.Builder mutateBuilder = MutationProto.newBuilder(proto);
+    mutateBuilder.setDurability(MutationProto.Durability.USE_DEFAULT);
+
+    Increment increment = ProtobufUtil.toIncrement(proto, null);
+    mutateBuilder.setTimestamp(increment.getTimestamp());
+    mutateBuilder.setTimeRange(ProtobufUtil.toTimeRange(increment.getTimeRange()));
+    assertEquals(mutateBuilder.build(), ProtobufUtil.toMutation(MutationType.INCREMENT, increment));
+  }
+
+  /**
+   * Older clients may not send along a timestamp in the MutationProto. Check that we
+   * default correctly.
+   */
+  @Test
+  public void testIncrementNoTimestamp() throws IOException {
+    MutationProto mutation = getIncrementMutation(null);
+    Increment increment = ProtobufUtil.toIncrement(mutation, null);
+    assertEquals(HConstants.LATEST_TIMESTAMP, increment.getTimestamp());
+    increment.getFamilyCellMap().values()
+      .forEach(cells ->
+        cells.forEach(cell ->
+          assertEquals(HConstants.LATEST_TIMESTAMP, cell.getTimestamp())));
+  }
+
+  private MutationProto getIncrementMutation(Long timestamp) {
     MutationProto.Builder mutateBuilder = MutationProto.newBuilder();
     mutateBuilder.setRow(ByteString.copyFromUtf8("row"));
     mutateBuilder.setMutateType(MutationType.INCREMENT);
@@ -213,61 +266,17 @@ public class TestProtobufUtil {
     valueBuilder.setFamily(ByteString.copyFromUtf8("f1"));
     QualifierValue.Builder qualifierBuilder = QualifierValue.newBuilder();
     qualifierBuilder.setQualifier(ByteString.copyFromUtf8("c1"));
-    qualifierBuilder.setValue(ByteStringer.wrap(Bytes.toBytes(11L)));
+    qualifierBuilder.setValue(ByteString.copyFrom(Bytes.toBytes(11L)));
+    if (timestamp != null) {
+      qualifierBuilder.setTimestamp(timestamp);
+    }
     valueBuilder.addQualifierValue(qualifierBuilder.build());
     qualifierBuilder.setQualifier(ByteString.copyFromUtf8("c2"));
-    qualifierBuilder.setValue(ByteStringer.wrap(Bytes.toBytes(22L)));
+    qualifierBuilder.setValue(ByteString.copyFrom(Bytes.toBytes(22L)));
     valueBuilder.addQualifierValue(qualifierBuilder.build());
     mutateBuilder.addColumnValue(valueBuilder.build());
 
-    MutationProto proto = mutateBuilder.build();
-    // default fields
-    assertEquals(MutationProto.Durability.USE_DEFAULT, proto.getDurability());
-
-    // set the default value for equal comparison
-    mutateBuilder = MutationProto.newBuilder(proto);
-    mutateBuilder.setDurability(MutationProto.Durability.USE_DEFAULT);
-
-    Increment increment = ProtobufUtil.toIncrement(proto, null);
-    assertEquals(mutateBuilder.build(),
-      ProtobufUtil.toMutation(increment, MutationProto.newBuilder(), HConstants.NO_NONCE));
-  }
-
-  @Test
-  public void testIncrementRequestConversion() throws Exception {
-    Increment inc = new Increment(Bytes.toBytes("row1"));
-    inc.addColumn(Bytes.toBytes("f1"), Bytes.toBytes("q1"), 10);
-    inc.addColumn(Bytes.toBytes("f1"), Bytes.toBytes("q2"), 20);
-    inc.addColumn(Bytes.toBytes("f2"), Bytes.toBytes("q3"), 30);
-    MutationProto proto =
-        ProtobufUtil.toMutation(inc, MutationProto.newBuilder(), HConstants.NO_NONCE);
-    Increment deSerializedIncrement = ProtobufUtil.toIncrement(proto, null);
-    List<Cell> cellList = deSerializedIncrement.getFamilyCellMap().get(Bytes.toBytes("f1"));
-    Collections.sort(cellList, KeyValue.COMPARATOR);
-    Cell cell0 = inc.getFamilyCellMap().get(Bytes.toBytes("f1")).get(0);
-    Cell cell1 = inc.getFamilyCellMap().get(Bytes.toBytes("f2")).get(0);
-    // We do not serialize TS for increment, hence need to compare parts of cell individually
-    // Compare cell0
-    assertEquals(Bytes.compareTo(cell0.getRowArray(), cell0.getRowOffset(), cell0.getRowLength(),
-      cellList.get(0).getRowArray(), cellList.get(0).getRowOffset(),
-      cellList.get(0).getRowLength()), 0);
-    assertEquals(Bytes.compareTo(cell0.getQualifierArray(), cell0.getQualifierOffset(),
-      cell0.getQualifierLength(), cellList.get(0).getQualifierArray(),
-      cellList.get(0).getQualifierOffset(), cellList.get(0).getQualifierLength()), 0);
-    assertEquals(Bytes.compareTo(cell0.getValueArray(), cell0.getValueOffset(),
-      cell0.getValueLength(), cellList.get(0).getValueArray(), cellList.get(0).getValueOffset(),
-      cellList.get(0).getValueLength()), 0);
-    // Compare cell1
-    Cell deSerCell = deSerializedIncrement.getFamilyCellMap().get(Bytes.toBytes("f2")).get(0);
-    assertEquals(Bytes.compareTo(cell1.getRowArray(), cell1.getRowOffset(), cell1.getRowLength(),
-      deSerCell.getRowArray(), deSerCell.getRowOffset(), deSerCell.getRowLength()), 0);
-    assertEquals(Bytes.compareTo(cell1.getQualifierArray(), cell1.getQualifierOffset(),
-      cell1.getQualifierLength(), deSerCell.getQualifierArray(), deSerCell.getQualifierOffset(),
-      deSerCell.getQualifierLength()), 0);
-    assertEquals(
-      Bytes.compareTo(cell1.getValueArray(), cell1.getValueOffset(), cell1.getValueLength(),
-        deSerCell.getValueArray(), deSerCell.getValueOffset(), deSerCell.getValueLength()),
-      0);
+    return mutateBuilder.build();
   }
 
   /**
@@ -306,7 +315,7 @@ public class TestProtobufUtil {
     // put value always use the default timestamp if no
     // value level timestamp specified,
     // add the timestamp to the original mutate
-    long timestamp = put.getTimeStamp();
+    long timestamp = put.getTimestamp();
     for (ColumnValue.Builder column:
         mutateBuilder.getColumnValueBuilderList()) {
       for (QualifierValue.Builder qualifier:
@@ -352,6 +361,7 @@ public class TestProtobufUtil {
     scanBuilder.setMaxVersions(2);
     scanBuilder.setCacheBlocks(false);
     scanBuilder.setCaching(1024);
+    scanBuilder.setTimeRange(ProtobufUtil.toTimeRange(TimeRange.allTime()));
     scanBuilder.setIncludeStopRow(false);
     ClientProtos.Scan expectedProto = scanBuilder.build();
 
@@ -360,117 +370,53 @@ public class TestProtobufUtil {
     assertEquals(expectedProto, actualProto);
   }
 
+  @Test
+  public void testToCell() throws Exception {
+    KeyValue kv1 =
+        new KeyValue(Bytes.toBytes("aaa"), Bytes.toBytes("f1"), Bytes.toBytes("q1"), new byte[30]);
+    KeyValue kv2 =
+        new KeyValue(Bytes.toBytes("bbb"), Bytes.toBytes("f1"), Bytes.toBytes("q1"), new byte[30]);
+    KeyValue kv3 =
+        new KeyValue(Bytes.toBytes("ccc"), Bytes.toBytes("f1"), Bytes.toBytes("q1"), new byte[30]);
+    byte[] arr = new byte[kv1.getLength() + kv2.getLength() + kv3.getLength()];
+    System.arraycopy(kv1.getBuffer(), kv1.getOffset(), arr, 0, kv1.getLength());
+    System.arraycopy(kv2.getBuffer(), kv2.getOffset(), arr, kv1.getLength(), kv2.getLength());
+    System.arraycopy(kv3.getBuffer(), kv3.getOffset(), arr, kv1.getLength() + kv2.getLength(),
+      kv3.getLength());
+    ByteBuffer dbb = ByteBuffer.allocateDirect(arr.length);
+    dbb.put(arr);
+    ByteBufferKeyValue offheapKV = new ByteBufferKeyValue(dbb, kv1.getLength(), kv2.getLength());
+    CellProtos.Cell cell = ProtobufUtil.toCell(offheapKV);
+    Cell newOffheapKV =
+        ProtobufUtil.toCell(ExtendedCellBuilderFactory.create(CellBuilderType.SHALLOW_COPY), cell);
+    assertTrue(CellComparatorImpl.COMPARATOR.compare(offheapKV, newOffheapKV) == 0);
+  }
+
   @Test
   public void testMetaRegionState() throws Exception {
     ServerName serverName = ServerName.valueOf("localhost", 1234, 5678);
     // New region state style.
     for (RegionState.State state: RegionState.State.values()) {
       RegionState regionState =
-          new RegionState(HRegionInfo.FIRST_META_REGIONINFO, state, serverName);
+          new RegionState(RegionInfoBuilder.FIRST_META_REGIONINFO, state, serverName);
       MetaRegionServer metars = MetaRegionServer.newBuilder()
-          .setServer(org.apache.hadoop.hbase.protobuf.ProtobufUtil.toServerName(serverName))
+          .setServer(org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil.toServerName(serverName))
           .setRpcVersion(HConstants.RPC_CURRENT_VERSION)
           .setState(state.convert()).build();
       // Serialize
       byte[] data = ProtobufUtil.prependPBMagic(metars.toByteArray());
       ProtobufUtil.prependPBMagic(data);
       // Deserialize
-      RegionState regionStateNew = ProtobufUtil.parseMetaRegionStateFrom(data, 1);
+      RegionState regionStateNew =
+          org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil.parseMetaRegionStateFrom(data, 1);
       assertEquals(regionState.getServerName(), regionStateNew.getServerName());
       assertEquals(regionState.getState(), regionStateNew.getState());
     }
     // old style.
     RegionState rs =
-        org.apache.hadoop.hbase.protobuf.ProtobufUtil.parseMetaRegionStateFrom(
+        org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil.parseMetaRegionStateFrom(
             serverName.getVersionedBytes(), 1);
     assertEquals(serverName, rs.getServerName());
     assertEquals(rs.getState(), RegionState.State.OPEN);
   }
-
-  /**
-   * Test {@link ProtobufUtil#toCell(Cell, boolean)} and
-   * {@link ProtobufUtil#toCell(CellProtos.Cell, boolean)} conversion
-   * methods when it contains tags and encode/decode tags is set to true.
-   */
-  @Test
-  public void testCellConversionWithTags() {
-    Cell cell = getCellWithTags();
-    CellProtos.Cell protoCell = ProtobufUtil.toCell(cell, true);
-    assertNotNull(protoCell);
-
-    Cell decodedCell = getCellFromProtoResult(protoCell, true);
-    List<Tag> decodedTags = Tag.asList(decodedCell.getTagsArray(), decodedCell.getTagsOffset(),
-        decodedCell.getTagsLength());
-    assertEquals(1,  decodedTags.size());
-    Tag decodedTag = decodedTags.get(0);
-    assertEquals(TAG_TYPE, decodedTag.getType());
-    assertEquals(TAG_STR, Bytes.toStringBinary(decodedTag.getValue()));
-  }
-
-  private Cell getCellWithTags() {
-    Tag tag = new Tag(TAG_TYPE, TAG_STR);
-    KeyValue kv = new KeyValue(Bytes.toBytes("row1"), Bytes.toBytes("f1"),
-      Bytes.toBytes("q1"), 10L, Bytes.toBytes("value1"), new Tag[] {tag});
-    return kv;
-  }
-
-  private Cell getCellFromProtoResult(CellProtos.Cell protoCell, boolean decodeTags) {
-    return ProtobufUtil.toCell(protoCell, decodeTags);
-  }
-
-  /**
-   * Test {@link ProtobufUtil#toCell(Cell, boolean)} and
-   * {@link ProtobufUtil#toCell(CellProtos.Cell, boolean)} conversion
-   * methods when it contains tags and encode/decode tags is set to false.
-   */
-  @Test
-  public void testCellConversionWithoutTags() {
-    Cell cell = getCellWithTags();
-    CellProtos.Cell protoCell =
-      ProtobufUtil.toCell(cell, false);
-    assertNotNull(protoCell);
-
-    Cell decodedCell = getCellFromProtoResult(protoCell, false);
-    List<Tag> decodedTags = Tag.asList(decodedCell.getTagsArray(), decodedCell.getTagsOffset(),
-      decodedCell.getTagsLength());
-    assertEquals(0,  decodedTags.size());
-  }
-
-  /**
-   * Test {@link ProtobufUtil#toCell(Cell, boolean)} and
-   * {@link ProtobufUtil#toCell(CellProtos.Cell, boolean)} conversion
-   * methods when it contains tags and encoding of tags is set to false
-   * and decoding of tags is set to true.
-   */
-  @Test
-  public void testTagEncodeFalseDecodeTrue() {
-    Cell cell = getCellWithTags();
-    CellProtos.Cell protoCell =
-      ProtobufUtil.toCell(cell, false);
-    assertNotNull(protoCell);
-
-    Cell decodedCell = getCellFromProtoResult(protoCell, true);
-    List<Tag> decodedTags = Tag.asList(decodedCell.getTagsArray(), decodedCell.getTagsOffset(),
-      decodedCell.getTagsLength());
-    assertEquals(0,  decodedTags.size());
-  }
-
-  /**
-   * Test {@link ProtobufUtil#toCell(Cell, boolean)} and
-   * {@link ProtobufUtil#toCell(CellProtos.Cell, boolean)} conversion
-   * methods when it contains tags and encoding of tags is set to true
-   * and decoding of tags is set to false.
-   */
-  @Test
-  public void testTagEncodeTrueDecodeFalse() {
-    Cell cell = getCellWithTags();
-    CellProtos.Cell protoCell =
-      ProtobufUtil.toCell(cell, true);
-    assertNotNull(protoCell);
-
-    Cell decodedCell = getCellFromProtoResult(protoCell, false);
-    List<Tag> decodedTags = Tag.asList(decodedCell.getTagsArray(), decodedCell.getTagsOffset(),
-      decodedCell.getTagsLength());
-    assertEquals(0,  decodedTags.size());
-  }
 }
