@@ -115,15 +115,18 @@ public class TestHStoreFile extends HBaseTestCase {
   }
 
   /**
-   * Write a file and then assert that we can read from top and bottom halves
-   * using two HalfMapFiles.
+   * Write a file and then assert that we can read from top and bottom halves using two
+   * HalfMapFiles, as well as one HalfMapFile and one HFileLink file.
    */
   @Test
-  public void testBasicHalfMapFile() throws Exception {
+  public void testBasicHalfAndHFileLinkMapFile() throws Exception {
     final HRegionInfo hri =
-        new HRegionInfo(TableName.valueOf("testBasicHalfMapFileTb"));
-    HRegionFileSystem regionFs = HRegionFileSystem.createRegionOnFileSystem(
-      conf, fs, new Path(testDir, hri.getTable().getNameAsString()), hri);
+        new HRegionInfo(TableName.valueOf("testBasicHalfAndHFileLinkMapFile"));
+    // The locations of HFileLink refers hfiles only should be consistent with the table dir
+    // create by CommonFSUtils directory, so we should make the region directory under
+    // the mode of CommonFSUtils.getTableDir here.
+    HRegionFileSystem regionFs = HRegionFileSystem.createRegionOnFileSystem(conf, fs,
+      CommonFSUtils.getTableDir(CommonFSUtils.getRootDir(conf), hri.getTable()), hri);
 
     HFileContext meta = new HFileContextBuilder().withBlockSize(2*1024).build();
     StoreFileWriter writer = new StoreFileWriter.Builder(conf, cacheConf, this.fs)
@@ -142,7 +145,7 @@ public class TestHStoreFile extends HBaseTestCase {
   }
 
   // pick an split point (roughly halfway)
-  byte[] SPLITKEY = new byte[] { (LAST_CHAR + FIRST_CHAR)/2, FIRST_CHAR};
+  byte[] SPLITKEY = new byte[] { (LAST_CHAR + FIRST_CHAR) / 2, FIRST_CHAR };
 
   /*
    * Writes HStoreKey and ImmutableBytes data to passed writer and
@@ -383,8 +386,10 @@ public class TestHStoreFile extends HBaseTestCase {
       throws IOException {
     f.initReader();
     Cell midkey = f.getReader().midKey().get();
-    KeyValue midKV = (KeyValue)midkey;
-    byte [] midRow = CellUtil.cloneRow(midKV);
+    KeyValue midKV = (KeyValue) midkey;
+    // 1. test using the midRow as the splitKey, this test will generate two Reference files
+    // in the children
+    byte[] midRow = CellUtil.cloneRow(midKV);
     // Create top split.
     HRegionInfo topHri = new HRegionInfo(regionFs.getRegionInfo().getTable(), null, midRow);
     Path topPath = splitStoreFile(regionFs, topHri, TEST_FAMILY, f, midRow, true);
@@ -446,7 +451,7 @@ public class TestHStoreFile extends HBaseTestCase {
       regionFs.cleanupDaughterRegion(topHri);
       regionFs.cleanupDaughterRegion(bottomHri);
 
-      // Next test using a midkey that does not exist in the file.
+      // 2. test using a midkey which will generate one Reference file and one HFileLink file.
       // First, do a key that is < than first key. Ensure splits behave
       // properly.
       byte [] badmidkey = Bytes.toBytes("  .");
