@@ -19,12 +19,12 @@
 
 require 'hbase_constants'
 
-include HBaseConstants
-
 module Hbase
   # Constructor tests
   class TableConstructorTest < Test::Unit::TestCase
     include TestHelpers
+    include HBaseConstants
+
     def setup
       setup_hbase
     end
@@ -97,6 +97,7 @@ module Hbase
   # Simple data management methods tests
   class TableSimpleMethodsTest < Test::Unit::TestCase
     include TestHelpers
+    include HBaseConstants
 
     def setup
       setup_hbase
@@ -225,6 +226,7 @@ module Hbase
   # rubocop:disable Metrics/ClassLength
   class TableComplexMethodsTest < Test::Unit::TestCase
     include TestHelpers
+    include HBaseConstants
 
     def setup
       setup_hbase
@@ -237,6 +239,7 @@ module Hbase
       @test_ts = 12345678
       @test_table.put(1, "x:a", 1)
       @test_table.put(1, "x:b", 2, @test_ts)
+      @test_table.put(1, "x:\x11", [921].pack("N"))
 
       @test_table.put(2, "x:a", 11)
       @test_table.put(2, "x:b", 12, @test_ts)
@@ -331,9 +334,10 @@ module Hbase
     end
 
     define_test "get should work with hash columns spec and an array of strings COLUMN parameter" do
-      res = @test_table._get_internal('1', COLUMN => [ 'x:a', 'x:b' ])
+      res = @test_table._get_internal('1', COLUMN => [ "x:\x11", 'x:a', 'x:b' ])
       assert_not_nil(res)
       assert_kind_of(Hash, res)
+      assert_not_nil(res['x:\x11'])
       assert_not_nil(res['x:a'])
       assert_not_nil(res['x:b'])
     end
@@ -354,6 +358,18 @@ module Hbase
       assert_not_nil(res['x:b'])
     end
 
+    define_test "get should work with non-printable columns and values" do
+      res = @test_table._get_internal('1', COLUMNS => [ "x:\x11" ])
+      assert_not_nil(res)
+      assert_kind_of(Hash, res)
+      assert_match(/value=\\x00\\x00\\x03\\x99/, res[ 'x:\x11' ])
+
+      res = @test_table._get_internal('1', COLUMNS => [ "x:\x11:toInt" ])
+      assert_not_nil(res)
+      assert_kind_of(Hash, res)
+      assert_match(/value=921/, res[ 'x:\x11' ])
+    end
+
     define_test "get should work with hash columns spec and TIMESTAMP only" do
       res = @test_table._get_internal('1', TIMESTAMP => @test_ts)
       assert_not_nil(res)
@@ -410,10 +426,10 @@ module Hbase
       assert_not_nil(res['x:b'])
     end
 
-    define_test "get with a block should yield (column, value) pairs" do
+    define_test "get with a block should yield (formatted column, value) pairs" do
       res = {}
       @test_table._get_internal('1') { |col, val| res[col] = val }
-      assert_equal(res.keys.sort, [ 'x:a', 'x:b' ])
+      assert_equal([ 'x:\x11', 'x:a', 'x:b' ], res.keys.sort)
     end
     
     define_test "get should support COLUMNS with value CONVERTER information" do
@@ -707,12 +723,14 @@ module Hbase
     define_test "scan should support COLUMNS with value CONVERTER information" do
       @test_table.put(1, "x:c", [1024].pack('N'))
       @test_table.put(1, "x:d", [98].pack('N'))
+      @test_table.put(1, "x:\x11", [712].pack('N'))
       begin
-        res = @test_table._scan_internal COLUMNS => ['x:c:toInt', 'x:d:c(org.apache.hadoop.hbase.util.Bytes).toInt']
+        res = @test_table._scan_internal COLUMNS => ['x:c:toInt', 'x:d:c(org.apache.hadoop.hbase.util.Bytes).toInt', "x:\x11:toInt"]
         assert_not_nil(res)
         assert_kind_of(Hash, res)
-        assert_not_nil(/value=1024/.match(res['1']['x:c']))
-        assert_not_nil(/value=98/.match(res['1']['x:d']))
+        assert_match(/value=1024/, res['1']['x:c'])
+        assert_match(/value=98/, res['1']['x:d'])
+        assert_match(/value=712/, res['1']['x:\x11'])
       ensure
         # clean up newly added columns for this test only.
         @test_table.deleteall(1, 'x:c')
